"""Adapter for ns-2.

This module contains the code for converting an FNSS topology object into a Tcl
script to deploy such topology into ns-2.
"""
from warnings import warn

from fnss.units import time_units, capacity_units
from fnss.netconfig.nodeconfig import get_stack, \
                                      get_application_names, \
                                      get_application_properties


__all__ = [
    'to_ns2',
    'validate_ns2_stacks'
          ]


# Template text rendered by the template engine
__TEMPLATE = r"""# Code generated by Fast Network Simulator Setup (FNSS)
<%
from fnss.units import time_units, capacity_units
from fnss.netconfig.nodeconfig import get_stack, \
                                      get_application_names, \
                                      get_application_properties

# Convert capacity in Mb
capacity_norm = capacity_units[topology.graph['capacity_unit']] / 1000000.0
# Convert delay it in ms
if set_delays: delay_norm = time_units[topology.graph['delay_unit']]
%>
# Create a simulator object
set ns [new Simulator]

# Create nodes
% for node in topology.nodes():
set n${str(node)} [$ns node]
% endfor

# Create all links
set qtype DropTail
## if topology is undirected, create duplex links, otherwise simplex links
% if topology.is_directed():
    % for u, v in topology.edges():
<% delay = "0" if not set_delays else str(topology.adj[u][v]['delay'] * delay_norm) %>\
${"$ns simplex-link $n%s $n%s %sMb %sms $qtype" % (str(u), str(v), str(topology.adj[u][v]['capacity'] * capacity_norm), delay)}
    % endfor
% else:
    % for u, v in topology.edges():
<% delay = "0" if not set_delays else str(topology.adj[u][v]['delay'] * delay_norm) %>\
${"$ns duplex-link $n%s $n%s %sMb %sms $qtype" % (str(u), str(v), str(topology.adj[u][v]['capacity'] * capacity_norm), delay)}
    % endfor
%endif

% if set_weights:
# Set link weights
    % for u, v in topology.edges():
${"$ns cost $n%s $n%s %s" % (str(u), str(v), str(topology.adj[u][v]['weight']))}
        % if not topology.is_directed():
${"$ns cost $n%s $n%s %s" % (str(v), str(u), str(topology.adj[v][u]['weight']))}
        % endif
    % endfor
% endif

% if set_buffers:
# Set queue sizes
    % for u, v in topology.edges():
${"$ns queue-limit $n%s $n%s %s" % (str(u), str(v), str(topology.adj[u][v]['buffer']))}
        % if not topology.is_directed():
${"$ns queue-limit $n%s $n%s %s" % (str(v), str(u), str(topology.adj[v][u]['buffer']))}
        % endif
    % endfor
% endif

% if deploy_stacks:
# Deploy applications and agents
    % for node in topology.nodes():
<%
stack = get_stack(topology, node)
if stack is None:
    continue
stack_name, stack_props = stack
stack_class = stack_props['class']
%>\
${"set %s [new %s]" % (str(stack_name), str(stack_class))}
        % for prop_name, prop_val in stack_props.items():
<% if prop_name == 'class': continue %>\
${"$%s set %s %s" % (str(stack_name), str(prop_name), str(prop_val))}
        % endfor
${"$ns attach-agent $n%s $%s" % (str(node), str(stack_name))}
        % for app_name in get_application_names(topology, node):
<%
app_properties = get_application_properties(topology, node, app_name)
app_class = app_properties['class']
%>\
${"set %s [new %s]" % (str(app_name), str(app_class))}
            % for prop_name, prop_val in app_properties.items():
<% if prop_name == 'class': continue %>\
${"$%s set %s %s" % (str(app_name), str(prop_name), str(prop_val))}
            % endfor
${"$%s attach-agent $%s" % (str(app_name), str(stack_name))}
        % endfor
    % endfor
% endif
"""


def validate_ns2_stacks(topology):
    """
    Validate whether the stacks and applications of a topology are valid for
    ns-2 deployment

    Parameters
    ----------
    topology : Topology
        The topology object to validate

    Returns
    -------
    valid : bool
        *True* if stacks are valid ns-2 stacks, *False* otherwise
    """
    for node in topology.nodes():
        applications = get_application_names(topology, node)
        for name in applications:
            if not 'class' in get_application_properties(topology, node, name):
                # Each application must have a class attribute to work
                return False
        stack = get_stack(topology, node)
        if stack is None:
            # Each node must have a stack if it has an application
            if len(applications) > 0: return False
        else:
            # If there is a stack, it must have a class attribute
            if 'class' not in stack[1]: return False
    return True


def to_ns2(topology, path, stacks=True):
    """Convert topology object into an ns-2 Tcl script that deploys that
    topology into ns-2.

    Parameters
    ----------
    topology : Topology
        The topology object to convert
    path : str
        The path to the output Tcl file
    stacks : bool, optional
        If True, read the stacks on nodes and write them into the output file.
        For this to work, stacks must be formatted in a way understandable by
        ns-2. For example, stack and applications must have a 'class' attribute
        whose value is the name of the ns-2 class implementing it.

    Notes
    -----
    In order for the function to parse stacks correctly, the input topology
    must satisfy the following requirements:
     * each stack and each application must have a `class` attribute whose
       value is the ns-2 class implementing such stack or application, such as
       `Agent/TCP` or `Application/FTP`.
     * All names and values of stack and application properties must be valid
       properties recognized by the ns-2 application or protocol stack.
    """
    try:
        from mako.template import Template
    except ImportError:
        raise ImportError('Cannot import mako.template module. '
                          'Make sure mako is installed on this machine.')
    set_buffers = True
    set_delays = True
    # if all links are annotated with weights, then set weights
    set_weights = all('weight' in topology.adj[u][v]
                      for u, v in topology.edges())

    if not 'capacity_unit' in topology.graph:
        raise ValueError('The given topology does not have capacity data.')
    if not topology.graph['capacity_unit'] in capacity_units:
        raise ValueError('The given topology does not have a valid capacity unit')
    if not 'buffer_unit' in topology.graph:
        warn('Missing buffer size unit attribute in the topology. '\
             'Output file will be generated without buffer assignments.')
        set_buffers = False
    elif not topology.graph['buffer_unit'] == 'packets':
        warn('The buffer size unit of the topology is %s. The only supported '
             'unit is packets. Output file will be generated without buffer '
             'assignments' % topology.graph['buffer_unit'])
        set_buffers = False
    if not 'delay_unit' in topology.graph or not topology.graph['delay_unit'] in time_units:
        warn('Missing or invalid delay unit attribute in the topology. The '
             'output file will be generated with all link delays set to 0.')
        set_delays = False
    if stacks:
        if not validate_ns2_stacks(topology):
            warn('Some application stacks cannot be parsed correctly. The '
                 'output file will be generated without stack assignments.')
            stacks = False
        if not any('stack' in topology.node[v] for v in topology.nodes()):
            stacks = False
    template = Template(__TEMPLATE)
    variables = {
        'topology':        topology,
        'deploy_stacks':   stacks,
        'set_delays':      set_delays,
        'set_buffers':     set_buffers,
        'set_weights':     set_weights
                }
    with open(path, "w") as out:
        out.write(template.render(**variables))



#!/usr/bin/env python
"""Run unit tests"""
import sys
from os import path, getcwd, environ, mkdir
from shutil import rmtree

def main():
    """Run all tests"""
    try:
        import pytest
    except ImportError:
        raise ImportError("The pytest package is needed to run the tests.")

    # get folder of Python source files
    src_dir = path.join(path.dirname(__file__), path.pardir)
    # get folder where resource files are stored and make it available to
    # test classes
    res_dir = path.abspath(path.join(path.dirname(__file__), 'resources'))
    tmp_dir = path.abspath(path.join(path.dirname(__file__), 'tmp'))
    if path.exists(res_dir):
        environ['test.res.dir'] = res_dir
    environ['test.tmp.dir'] = tmp_dir
    # stop if running from source directory
    if getcwd() == path.abspath(path.join(src_dir, path.pardir)):
        raise RuntimeError("Can't run tests from source directory.\n"
                           "Run 'py.test' from the command line.")

    # Prepare tests
    if path.exists(tmp_dir):
        rmtree(tmp_dir)
    mkdir(tmp_dir)

    # Run tests
    res = pytest.main(['.'])

    # Clean up
    if path.exists(tmp_dir):
        rmtree(tmp_dir)

    # Return results
    return res

if __name__ == "__main__":
    sys.exit(main())



from os import path, environ
import unittest

import fnss

TMP_DIR = environ['test.tmp.dir'] if 'test.tmp.dir' in environ else None


class Test(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        # set up topology used for all traffic matrix tests
        cls.G = fnss.glp_topology(n=50, m=1, m0=10, p=0.2, beta=-2, seed=1)
        fnss.set_capacities_random(cls.G, {10: 0.5, 20: 0.3, 40: 0.2},
                              capacity_unit='Mbps')
        fnss.set_delays_constant(cls.G, 2, delay_unit='ms')
        fnss.set_weights_inverse_capacity(cls.G)
        for node in [2, 4, 6]:
            fnss.add_stack(cls.G, node, 'tcp',
                          {'protocol': 'cubic', 'rcvwnd': 1024})
        for node in [2, 4]:
            fnss.add_application(cls.G, node, 'client',
                                {'rate': 100, 'user-agent': 'fnss'})
        fnss.add_application(cls.G, 2, 'server',
                           {'port': 80, 'active': True, 'user-agent': 'fnss'})

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_base_topology_class(self):
        weight = 2
        capacity = 3
        delay = 4
        buffer_size = 5
        topology = fnss.Topology()
        topology.add_path([1, 2, 3])
        fnss.set_weights_constant(topology, weight)
        fnss.set_capacities_constant(topology, capacity)
        fnss.set_delays_constant(topology, delay)
        fnss.set_buffer_sizes_constant(topology, buffer_size)
        weights = topology.weights()
        capacities = topology.capacities()
        delays = topology.delays()
        buffer_sizes = topology.buffers()
        for e in topology.edges():
            self.assertEqual(weight, weights[e])
            self.assertEqual(capacity, capacities[e])
            self.assertEqual(delay, delays[e])
            self.assertEqual(buffer_size, buffer_sizes[e])

    def test_topology_class(self):
        topology = fnss.Topology()
        topology.add_edge(1, 2)
        self.assertEqual(1, topology.number_of_edges())
        topology.add_edge(2, 1)
        self.assertEqual(1, topology.number_of_edges())
        topology.add_edge('1', '2')
        topology.add_edge('2', '1')
        self.assertEqual(2, topology.number_of_edges())

    def test_directed_topology_class(self):
        topology = fnss.DirectedTopology()
        topology.add_edge(1, 2)
        topology.add_edge(2, 1)
        self.assertEqual(2, topology.number_of_edges())

    def test_od_pairs_from_topology_directed(self):
        dir_topology = fnss.DirectedTopology()
        dir_topology.add_edge(0, 1)
        dir_topology.add_edge(1, 0)
        dir_topology.add_edge(1, 2)
        dir_topology.add_edge(3, 2)
        dir_topology.add_edge(8, 9)
        expected_od_pairs = [(0, 1), (0, 2), (1, 0), (1, 2), (3, 2), (8, 9)]
        od_pairs = fnss.od_pairs_from_topology(dir_topology)
        self.assertEquals(len(expected_od_pairs), len(od_pairs))
        for od in expected_od_pairs:
            self.assertTrue(od in od_pairs)

    def test_od_pairs_from_topology_undirected(self):
        topology = fnss.ring_topology(3)
        topology.add_path([7, 8, 9])  # isolated node: no flows from/to this node
        od_pairs = fnss.od_pairs_from_topology(topology)
        expected_od_pairs = [(0, 1), (0, 2), (1, 0), (1, 2), (2, 0), (2, 1),
                             (7, 8), (7, 9), (8, 7), (8, 9), (9, 7), (9, 8)]
        self.assertEquals(len(expected_od_pairs), len(od_pairs))
        for od in expected_od_pairs:
            self.assertTrue(od in od_pairs)

    def test_fan_in_out_capacities_directed(self):
        dir_topology = fnss.DirectedTopology()
        dir_topology.add_edge(0, 1)
        dir_topology.add_edge(1, 0)
        dir_topology.add_edge(1, 2)
        dir_topology.add_edge(3, 2)
        fnss.set_capacities_constant(dir_topology, 10, 'Mbps')
        in_cap, out_cap = fnss.fan_in_out_capacities(dir_topology)
        self.assertEquals({0: 10, 1: 10, 2: 20, 3: 0}, in_cap)
        self.assertEquals({0: 10, 1: 20, 2: 0, 3: 10}, out_cap)

    def test_fan_in_out_capacities_undirected(self):
        topology = fnss.star_topology(3)
        fnss.set_capacities_constant(topology, 10, 'Mbps')
        in_cap, out_cap = fnss.fan_in_out_capacities(topology)
        self.assertEquals({0: 30, 1: 10, 2: 10, 3: 10}, in_cap)
        self.assertEquals(in_cap, out_cap)

    @unittest.skipIf(TMP_DIR is None, "Temp folder not present")
    def test_read_write_topology(self):
        tmp_topo_file = path.join(TMP_DIR, 'toporw.xml')
        fnss.write_topology(self.G, tmp_topo_file)
        self.assertTrue(path.exists(tmp_topo_file))
        read_topo = fnss.read_topology(tmp_topo_file)
        self.assertEquals(len(self.G), len(read_topo))
        self.assertEquals(self.G.number_of_edges(),
                          read_topo.number_of_edges())
        self.assertEquals('tcp', fnss.get_stack(read_topo, 2)[0])
        self.assertEquals(1024, fnss.get_stack(read_topo, 2)[1]['rcvwnd'])
        self.assertEquals('cubic', fnss.get_stack(read_topo, 2)[1]['protocol'])
        self.assertEquals(len(fnss.get_application_names(self.G, 2)),
                      len(fnss.get_application_names(read_topo, 2)))
        self.assertEquals('fnss', fnss.get_application_properties(read_topo, 2, 'server')['user-agent'])
        self.assertEquals([2, 4, 6], [ v for v in read_topo.nodes()
                                      if fnss.get_stack(read_topo, v) is not None
                                      and fnss.get_stack(read_topo, v)[0] == 'tcp'])
        self.assertEquals([2, 4], [ v for v in read_topo.nodes()
                                   if 'client' in fnss.get_application_names(read_topo, v)])
        self.assertEquals([2], [ v for v in read_topo.nodes()
                                if 'server' in fnss.get_application_names(read_topo, v)])



import unittest

import fnss


class Test(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_line_topology(self):
        def test_line_connectivity(n):
            G = fnss.line_topology(n)
            self.assertEquals(n, G.number_of_nodes())
            self.assertEquals(n - 1, G.number_of_edges())
            for i in range(n):
                if i <= n - 2: self.assertTrue(G.has_edge(i, i + 1))
                if i >= 1:   self.assertTrue(G.has_edge(i, i - 1))
        self.assertRaises(ValueError, fnss.line_topology, 0)
        self.assertRaises(ValueError, fnss.line_topology, -1)
        test_line_connectivity(8)
        test_line_connectivity(11)

    def test_k_ary_tree_topology(self):
        def test_K_ary_tree_connectivity(k, h):
            expected_degree = {'root': k, 'intermediate': k + 1, 'leaf': 1}
            G = fnss.k_ary_tree_topology(k, h)
            self.assertEquals(sum(k ** d for d in range(h + 1)),
                              G.number_of_nodes())
            self.assertEquals(sum(k ** d for d in range(1, h + 1)),
                              G.number_of_edges())
            degree = G.degree()
            for v in G.nodes():
                v_type = G.node[v]['type']
                v_depth = G.node[v]['depth']
                self.assertEqual(expected_degree[v_type], degree[v])
                neighbors = G.neighbors(v)
                for u in neighbors:
                    u_depth = G.node[u]['depth']
                    if u < v:
                        self.assertEqual(u_depth, v_depth - 1)
                    elif u > v:
                        self.assertEqual(u_depth, v_depth + 1)
                    else:  # u == v
                        self.fail("Node %s has a self-loop" % str(v))
        self.assertRaises(ValueError, fnss.k_ary_tree_topology, 0, 3)
        self.assertRaises(ValueError, fnss.k_ary_tree_topology, 3, 0)
        self.assertRaises(ValueError, fnss.k_ary_tree_topology, -1, 3)
        self.assertRaises(ValueError, fnss.k_ary_tree_topology, 3, -1)
        test_K_ary_tree_connectivity(3, 5)
        test_K_ary_tree_connectivity(5, 3)
        test_K_ary_tree_connectivity(2, 1)

    def test_ring_topology(self):
        def test_ring_connectivity(n):
            G = fnss.ring_topology(n)
            self.assertEquals(n, G.number_of_nodes())
            self.assertEquals(n, G.number_of_edges())
            for i in range(n):
                self.assertTrue(G.has_edge(i, (i + 1) % n))
                self.assertTrue(G.has_edge(i, (i - 1) % n))
        self.assertRaises(ValueError, fnss.ring_topology, 0)
        self.assertRaises(ValueError, fnss.ring_topology, -1)
        self.assertRaises(TypeError, fnss.ring_topology, 'String')
        test_ring_connectivity(10)
        test_ring_connectivity(21)

    def test_star_topology(self):
        def test_star_connectivity(n):
            G = fnss.star_topology(n)
            self.assertEquals(n + 1, G.number_of_nodes())
            self.assertEquals(n, G.number_of_edges())
            self.assertEquals('root', G.node[0]['type'])
            for i in range(1, n + 1):
                self.assertEquals('leaf', G.node[i]['type'])
                self.assertTrue(G.has_edge(i, 0))
                self.assertTrue(G.has_edge(0, i))
        self.assertRaises(ValueError, fnss.star_topology, 0)
        self.assertRaises(ValueError, fnss.star_topology, -1)
        self.assertRaises(TypeError, fnss.star_topology, 'String')
        test_star_connectivity(10)
        test_star_connectivity(21)

    def test_full_mesh_topology(self):
        def test_full_mesh_connectivity(n):
            G = fnss.full_mesh_topology(n)
            self.assertEquals(n, G.number_of_nodes())
            self.assertEquals((n * (n - 1)) // 2, G.number_of_edges())
            for i in range(n):
                for j in range(n):
                    if i != j:
                        self.assertTrue(G.has_edge(i, j))
        self.assertRaises(ValueError, fnss.full_mesh_topology, 0)
        self.assertRaises(ValueError, fnss.full_mesh_topology, -1)
        self.assertRaises(TypeError, fnss.full_mesh_topology, 'String')
        test_full_mesh_connectivity(10)
        test_full_mesh_connectivity(21)

    def test_dumbbell_topology(self):
        def test_dumbbell_connectivity(m, n):
            G = fnss.dumbbell_topology(m, n)
            self.assertEquals(2 * m + n, G.number_of_nodes())
            self.assertEquals(2 * m + n - 1, G.number_of_edges())
            for i in range(m):
                self.assertTrue(G.has_edge(i, m))
                self.assertEquals('left_bell', G.node[i]['type'])
            for i in range(m, m + n):
                self.assertTrue(G.has_edge(i, i + 1))
                self.assertEquals('core', G.node[i]['type'])
            for i in range(m + n, 2 * m + n):
                self.assertTrue(G.has_edge(m + n - 1, i))
                self.assertEquals('right_bell', G.node[i]['type'])
        self.assertRaises(ValueError, fnss.dumbbell_topology, 0, 0)
        self.assertRaises(ValueError, fnss.dumbbell_topology, -1, 1)
        self.assertRaises(ValueError, fnss.dumbbell_topology, 1, 3)
        self.assertRaises(TypeError, fnss.dumbbell_topology, 'String', 4)
        self.assertRaises(TypeError, fnss.dumbbell_topology, 4, 'String')
        test_dumbbell_connectivity(15, 12)
        test_dumbbell_connectivity(2, 1)

    def test_chord_topology(self):
        def test_chord_connectivity(m, r):
            G = fnss.chord_topology(m, r)
            n = 2 ** m
            self.assertEqual(len(G), n)
            if r <= 2:
                for i in G.nodes():
                    self.assertEqual(len(G.adj[i]), m)
            else:
                for i in G.nodes():
                    for j in range(i + 1, i + r + 1):
                        self.assertTrue(G.has_edge(i, j % n))
        test_chord_connectivity(2, 1)
        test_chord_connectivity(3, 1)
        test_chord_connectivity(4, 1)
        test_chord_connectivity(5, 1)
        test_chord_connectivity(5, 2)
        test_chord_connectivity(5, 3)
        test_chord_connectivity(3, 7)

        self.assertRaises(ValueError, fnss.chord_topology, 0, 3)
        self.assertRaises(ValueError, fnss.chord_topology, 1, 3)
        self.assertRaises(ValueError, fnss.chord_topology, -1, 3)
        self.assertRaises(ValueError, fnss.chord_topology, 5, -1)
        self.assertRaises(ValueError, fnss.chord_topology, 5, 0)
        self.assertRaises(ValueError, fnss.chord_topology, 3, 8)

        self.assertRaises(TypeError, fnss.chord_topology, 5, None)
        self.assertRaises(TypeError, fnss.chord_topology, None, 3)
        self.assertRaises(TypeError, fnss.chord_topology, 5, "1")



"""Functions to assign and manipulate link delays."""
import networkx as nx
from fnss.units import time_units, distance_units

__all__ = [
    'PROPAGATION_DELAY_VACUUM',
    'PROPAGATION_DELAY_FIBER',
    'set_delays_constant',
    'set_delays_geo_distance',
    'get_delays',
    'clear_delays'
           ]

# Propagation delay of light in the vacuum
PROPAGATION_DELAY_VACUUM = 1.0 / 300  # ms/Km

# Propagation delay of light in an average optical fiber
PROPAGATION_DELAY_FIBER = 0.005  # ms/Km


def set_delays_constant(topology, delay=1.0, delay_unit='ms', links=None):
    """
    Assign a constant delay to all selected links

    Parameters
    ----------
    topology : Topology
        The topology on which delays are applied.
    delay : float, optional
        The constant delay to be applied to all links
    delay_unit : string, optional
        The unit of delays. Supported units are: "us" (microseconds), "ms"
        (milliseconds) and "s" (seconds)
    links : list, optional
        List of selected links on which weights are applied. If it is None,
        all links are selected

    Examples
    --------
    >>> import fnss
    >>> topology = fnss.Topology()
    >>> topology.add_path([1, 2, 4, 5, 8])
    >>> fnss.set_delays_constant(topology, 5.0, 'ms', links=[(1,2), (5,8), (4,5)])
    >>> delay = fnss.get_delays(topology)
    >>> delay[(1, 2)]
    5.0
    """
    if not delay_unit in time_units:
        raise ValueError("The delay_unit argument is not valid")
    conversion_factor = 1
    if 'delay_unit' in topology.graph and links is not None:
        # If a delay_unit is set, that means that some links have already
        # been assigned delays, so set these delay using the same unit
        # already used
        curr_delay_unit = topology.graph['delay_unit']
        if curr_delay_unit != delay_unit:
            conversion_factor = float(time_units[delay_unit]) \
                                / time_units[curr_delay_unit]
    else:
        topology.graph['delay_unit'] = delay_unit
    edges = links or topology.edges()
    for u, v in edges:
        topology.adj[u][v]['delay'] = delay * conversion_factor


def set_delays_geo_distance(topology, specific_delay, default_delay=None,
                            delay_unit='ms', links=None):
    """
    Assign a delay to all selected links equal to the product of link length
    and specific delay. To use this function, all nodes must have a 'latitude'
    and a 'longitude' attribute. Alternatively, all links of the topology must
    have a 'length' attribute. If the length of a link cannot be determined, it
    is applied the delay equal default_delay if specified, otherwise an error
    is returned.

    Parameters
    ----------
    topology : Topology
        The topology on which delays are applied.
    specific_delay : float
        The specific delay (in ms/Km) to be applied to all links
    default_delay : float, optional
        The delay to be applied to links whose length is not known. If None, if
        the length of a link cannot be determined, an error is returned
    delay_unit : string, optional
        The unit of delays. Supported units are: "us" (microseconds), "ms"
        (milliseconds) and "s" (seconds)
    links : list, optional
        List of selected links on which weights are applied. If it is None, all
        links are selected

    Examples
    --------
    >>> import fnss
    >>> topology = fnss.parse_abilene('abilene_topo.txt')
    >>> fnss.set_delays_geo_distance(topology, specific_delay=fnss.PROPAGATION_DELAY_FIBER)
    """
    # Validate input parameters
    if not delay_unit in time_units:
        raise ValueError("The delay_unit argument is not valid")
    if not 'distance_unit' in topology.graph:
        raise ValueError("The provided topology does not have a "\
                         "distance_unit attribute")
    distance_unit = topology.graph['distance_unit']
    if distance_unit not in distance_units:
        raise ValueError("The distance_unit attribute of the provided "\
                         "topology (%s) is not valid" % distance_unit)
    edges = links or topology.edges()
    if default_delay is None:
        if any(('length' not in topology.adj[u][v] for u, v in edges)):
            raise ValueError('All links must have a length attribute')
    if 'delay_unit' in topology.graph and links is not None:
        # If a delay_unit is set, that means that some links have already
        # been assigned delays, so set these delays using the same unit
        # already used instead of the delay unit provided as argument
        curr_delay_unit = topology.graph['delay_unit']
        conv_factor = 1.0 / time_units[curr_delay_unit]
    else:
        topology.graph['delay_unit'] = delay_unit
        curr_delay_unit = delay_unit  # used in case of default delay assignment
        conv_factor = 1.0 / time_units[delay_unit]
    # factor to convert length value in Km
    length_conv_factor = distance_units[distance_unit]
    # factor to convert default delay in target delay unit
    default_conv_factor = time_units[delay_unit] / time_units[curr_delay_unit]
    for u, v in edges:
        if 'length' in topology.adj[u][v]:
            length = topology.adj[u][v]['length'] * length_conv_factor
            delay = specific_delay * length * conv_factor
        else:
            delay = default_delay * default_conv_factor
        topology.adj[u][v]['delay'] = delay


def get_delays(topology):
    """
    Returns all the delays.

    Parameters
    ----------
    topology : Topology
        The topology whose link delays are requested

    Returns
    -------
    delays : dict
        Dictionary of link delays keyed by link.

    Examples
    --------
    >>> import fnss
    >>> topology = fnss.Topology()
    >>> topology.add_path([1,2,3])
    >>> fnss.set_delays_constant(topology, 10, 'ms')
    >>> delay = get_delays(topology)
    >>> delay[(1,2)]
    10
    """
    return nx.get_edge_attributes(topology, 'delay')


def clear_delays(topology):
    """
    Remove all delays from the topology.

    Parameters
    ----------
    topology : Topology
    """
    topology.graph.pop('delay_unit', None)
    for u, v in topology.edges():
        topology.adj[u][v].pop('delay', None)



from os import environ, path
import unittest

import fnss

TMP_DIR = environ['test.tmp.dir'] if 'test.tmp.dir' in environ else None

class Test(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    @unittest.skipIf(TMP_DIR is None, "Temp folder not present")
    def test_to_omnetpp_undirected(self):
        t = fnss.Topology()
        t.add_path([1, 2, 3, 4])
        fnss.set_capacities_constant(t, 10, 'Gbps')
        fnss.set_delays_constant(t, 2, 'us')
        fnss.set_buffer_sizes_constant(t, 20, 'packets')
        fnss.to_omnetpp(t, path.join(TMP_DIR, 'omnetpp-undir.ned'))

    @unittest.skipIf(TMP_DIR is None, "Temp folder not present")
    def test_to_omnetpp_directed(self):
        t = fnss.DirectedTopology()
        t.add_path([1, 2, 3, 4])
        fnss.set_capacities_constant(t, 10, 'Gbps')
        fnss.set_delays_constant(t, 2, 'us')
        fnss.set_buffer_sizes_constant(t, 20, 'packets')
        fnss.to_omnetpp(t, path.join(TMP_DIR, 'omnetpp-dir.ned'))



"""
Create datacenter topology for Mininet
======================================

This example shows how to create a datacenter topology with FNSS and export it
to Mininet.

This example requires Mininet to be installed on the machine.
"""
import fnss

from mininet.topo import Topo
from mininet.net import Mininet
from mininet.link import TCLink
from mininet.util import dumpNodeConnections
from mininet.log import setLogLevel
from mininet.node import OVSController


# Create FNSS topology. Let's create a simple datacenter topology
# This topology does not contain loops. If you want to use a topology with
# loops or multiple paths in Mininet you need to use a custom controller.
# More info here:
# https://github.com/mininet/mininet/wiki/Introduction-to-Mininet#multipath-routing
fnss_topo = fnss.two_tier_topology(n_core=1, n_edge=2, n_hosts=2)

# Set link attributes
fnss.set_capacities_constant(fnss_topo, 10, 'Mbps')
fnss.set_delays_constant(fnss_topo, 2, 'ms')
fnss.set_buffer_sizes_constant(fnss_topo, 50, 'packets')

# Convert FNSS topology to Mininet
# If argument relabel_nodes is set to False, node labels are not changed when
# converting an FNSS topology to a Mininet one, except converting the type to
# string (e.g. 1 -> '1'). If relabel_nodes is set to True (default option)
# then nodes are label according to Mininet conventions, e.g. hosts are
# prepended an h (e.g. 1 -> 'h1') and switches are prepended an s
# (e.g. 2 -> 's2')
mn_topo = fnss.to_mininet(fnss_topo, relabel_nodes=True)

# Create a Mininet instance and start it
# Use TCLink to implement links enables Linux Traffic Container (TC) for rate
# limitation
net = Mininet(topo=mn_topo, link=TCLink, controller=OVSController)
net.start()

# Dump host connections
dumpNodeConnections(net.hosts)

# Test network connectivity
net.pingAll()

# Test bandwidth between nodes
h1, h4 = net.get('h1', 'h4')
net.iperf((h1, h4))

# Stop Mininet
net.stop()



"""
Create dumbbell topology for Mininet
====================================

This example shows how to create a dumbbell topology with FNSS and export it
to Mininet.

This example requires Mininet to be installed on the machine.
"""
import networkx as nx
import fnss

from mininet.topo import Topo
from mininet.net import Mininet
from mininet.link import TCLink
from mininet.util import dumpNodeConnections
from mininet.log import setLogLevel
from mininet.node import OVSController


# Create FNSS topology. Let's create a simple dumbbell topology
# This topology does not contain loops. If you want to use a topology with
# loops or multiple paths in Mininet you need to use a custom controller.
# More info here:
# https://github.com/mininet/mininet/wiki/Introduction-to-Mininet#multipath-routing
fnss_topo = fnss.dumbbell_topology(5, 4)

# now extract links located in the edges and links in the core
# we do this easily using Python list comprehension. Each link in the dumbbell
# topology has a 'type' attribute which identifies whether it a link of the
# core path or it is in a bell.
# Look at dumbbell_topology documentation for more details.

# this return a dictionary of egdes and value of attribute type
# This function is provided by the NetworkX library.
# Since FNSS Topology and DirecteedTopology objects inherit from NetworkX's
# Graph and DiGraph, respectively, NetworkX functions can be used in FNSS too.
link_types = nx.get_edge_attributes(fnss_topo, 'type')
core_links = [links for links in link_types if link_types[links] == 'core']
edge_links = [links for links in link_types
              if link_types[links] == 'right_bell'
              or link_types[links] == 'left_bell']

# set delay equal to 1 ms in edge links and equal to 2 ms in core links
fnss.set_delays_constant(fnss_topo, 1, 'ms', edge_links)
fnss.set_delays_constant(fnss_topo, 2, 'ms', core_links)

# set capacity of 10 Mbps in edge links and 40 Mbps in core links
fnss.set_capacities_constant(fnss_topo, 10, 'Mbps', edge_links)
fnss.set_capacities_constant(fnss_topo, 40, 'Mbps', core_links)

# Set buffer size constant to all interfaces
fnss.set_buffer_sizes_constant(fnss_topo, 50, 'packets')

# Now we deploy a traffic sources on right bell and traffic receivers on left
# bell
node_types = nx.get_node_attributes(fnss_topo, 'type')
core_nodes = [nodes for nodes in node_types
              if node_types[nodes] == 'core']
left_nodes = [nodes for nodes in node_types
              if node_types[nodes] == 'left_bell']
right_nodes = [nodes for nodes in node_types
              if node_types[nodes] == 'right_bell']

# Set nodes on bells to be hosts and nodes on bottleneck path to be switches.
# Differently from datacenter topologies, which already provide annotation of
# what nodes are hosts and switches, we need to explicitly tell which nodes
# are switches and which are hosts to deploy the topology in Mininet
hosts = left_nodes + right_nodes
switches = core_nodes

# Convert FNSS topology to Mininet
# If argument relabel_nodes is set to False, node labels are not changed when
# converting an FNSS topology to a Mininet one, except converting the type to
# string (e.g. 1 -> '1'). If relabel_nodes is set to True (default option)
# then nodes are label according to Mininet conventions, e.g. hosts are
# prepended an h (e.g. 1 -> 'h1') and switches are prepended an s
# (e.g. 2 -> 's2')
mn_topo = fnss.to_mininet(fnss_topo,
                          switches=switches,
                          hosts=hosts,
                          relabel_nodes=True)

# Create a Mininet instance and start it
# Use TCLink to implement links enables Linux Traffic Container (TC) for rate
# limitation
net = Mininet(topo=mn_topo, link=TCLink, controller=OVSController)
net.start()

# Dump host connections
dumpNodeConnections(net.hosts)

# Test network connectivity
net.pingAll()

# Test bandwidth between nodes
h1, h8 = net.get('h1', 'h8')
net.iperf((h1, h8))

# Stop Mininet
net.stop()



"""
This module contains all the information related to the current release of the
library including descriptions, version number, authors and contact
information.
"""
# author information.
# Used by __init__, doc and setup
author = 'Lorenzo Saino, Cosmin Cocora'
author_email = 'fnss.dev@gmail.com'

# version information
# Used by __init__, doc and setup
version = '0.8.0'

# License information
# Used by __init__, doc and setup
license_short = 'BSD'
license_long = 'BSD license'

description_short = 'Fast creation and configuration of topologies, traffic ' \
                    'matrices and event schedules for network experiments'

description_long = """The Fast Network Simulation Setup (FNSS) core library is
a Python library providing a set of features allowing network researchers and
engineers to simplify the setup of a network experiment.

These features include the ability to:

* Parse a topology from a dataset, a topology generator or generate it
  according to a number of synthetic models.
* Apply link capacities, link weights, link delays and buffer sizes.
* Deploy protocol stacks and applications on network nodes.
* Generate traffic matrices.
* Generate event schedules.

The core library allows users to export the generated scenarios (topologies,
traffic matrices and event schedules) to ns-2, Mininet or AutoNetKit.

It also allows to save scenarios in XML files, which can be later imported
by the FNSS Java, C++ and ns-3 libraries.
"""

# URL
url = 'http://fnss.github.io/'
download_url = url



"""
Export topology to ns-2
=======================

This example shows how to generate a topology (a line in this case) and export
it to the ns-2 simulator
"""
import fnss

# create a line topology with 10 nodes
topology = fnss.line_topology(10)

# assign capacity of 10 Mbps to each link
fnss.set_capacities_constant(topology, 10, 'Mbps')

# assign delay of 2 ms to each link
fnss.set_delays_constant(topology, 2, 'ms')

# set buffers in each node (use packets, bytes not supported by ns-2)
fnss.set_buffer_sizes_bw_delay_prod(topology, 'packets', 1500)

# Add FTP application to first and last node of the line
tcp_stack_props = {'class': 'Agent/TCP', 'class_': 2, 'fid_': 1}
fnss.add_stack(topology, 0, 'tcp', tcp_stack_props)
fnss.add_stack(topology, 9, 'tcp', tcp_stack_props)

ftp_app_props = {'class': 'Application/FTP', 'type': 'FTP'}
fnss.add_application(topology, 0, 'ftp', ftp_app_props)
fnss.add_application(topology, 9, 'ftp', ftp_app_props)

# export topology to a Tcl script for ns-2
fnss.to_ns2(topology, 'ns2-script.tcl', stacks=True)



import unittest

import fnss

class Test(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        # set up topology used for all traffic matrix tests
        cls.topo = fnss.full_mesh_topology(100)
        cls.stack_1_name = 'stack_1'
        cls.stack_1_props = {'prop1': 'val11', 'prop2': 'val12'}
        cls.stack_2_name = 'stack_2'
        cls.stack_2_props = {'prop1': 'val12', 'prop2': 'val22'}
        cls.app_1_name = 'app_1'
        cls.app_1_props = {'prop1': 'val11', 'prop2': 'val12'}
        cls.app_2_name = 'app_2'
        cls.app_2_props = {'prop1': 'val12', 'prop2': 'val22'}

    def setUp(self):
        pass

    def tearDown(self):
        fnss.clear_stacks(self.topo)
        fnss.clear_applications(self.topo)

    def test_add_stack_no_attr(self):
        fnss.add_stack(self.topo, 1, 's_name')
        self.assertEqual(fnss.get_stack(self.topo, 1, data=False), 's_name')
        self.assertEqual(fnss.get_stack(self.topo, 1, data=True), ('s_name', {}))

    def test_add_stack_attr_dict(self):
        fnss.add_stack(self.topo, 1, 's_name', {'att1': 'val1'})
        self.assertEqual(fnss.get_stack(self.topo, 1, data=False), 's_name')
        self.assertEqual(fnss.get_stack(self.topo, 1, data=True), ('s_name', {'att1': 'val1'}))

    def test_add_stack_kw_attr(self):
        fnss.add_stack(self.topo, 1, 's_name', att1='val1')
        self.assertEqual(fnss.get_stack(self.topo, 1, data=False), 's_name')
        self.assertEqual(fnss.get_stack(self.topo, 1, data=True), ('s_name', {'att1': 'val1'}))

    def test_add_stack_mixed_attr(self):
        fnss.add_stack(self.topo, 1, 's_name', {'att1': 'val1'}, att2='val2')
        self.assertEqual(fnss.get_stack(self.topo, 1, data=False), 's_name')
        self.assertEqual(fnss.get_stack(self.topo, 1, data=True), ('s_name', {'att1': 'val1', 'att2': 'val2'}))

    def test_add_get_remove_stack(self):
        for v in self.topo.nodes():
            self.assertIsNone(fnss.get_stack(self.topo, v))
        fnss.add_stack(self.topo, 12, self.stack_1_name, self.stack_1_props)
        self.assertEqual(2, len(fnss.get_stack(self.topo, 12)))
        self.assertIsNone(fnss.get_stack(self.topo, 3))
        self.assertEqual(self.stack_1_name, fnss.get_stack(self.topo, 12)[0])
        self.assertEqual(self.stack_1_props, fnss.get_stack(self.topo, 12)[1])
        fnss.add_stack(self.topo, 12, self.stack_1_name, self.stack_2_props)
        self.assertEqual(self.stack_1_name, fnss.get_stack(self.topo, 12)[0])
        self.assertEqual(self.stack_2_props, fnss.get_stack(self.topo, 12)[1])
        fnss.add_stack(self.topo, 12, self.stack_2_name, self.stack_2_props)
        self.assertEqual(self.stack_2_name, fnss.get_stack(self.topo, 12)[0])
        self.assertEqual(self.stack_2_props, fnss.get_stack(self.topo, 12)[1])
        fnss.remove_stack(self.topo, 12)
        self.assertIsNone(fnss.get_stack(self.topo, 12))

    def test_clear_stacks(self):
        for v in self.topo.nodes():
            fnss.add_stack(self.topo, v, self.stack_1_name, self.stack_1_props)
        fnss.clear_stacks(self.topo)
        for v in self.topo.nodes():
            self.assertIsNone(fnss.get_stack(self.topo, v))

    def test_add_get_remove_applications(self):
        for v in self.topo.nodes():
            self.assertEqual([], fnss.get_application_names(self.topo, v))
        fnss.add_application(self.topo, 10, self.app_1_name, self.app_1_props)
        self.assertEqual([self.app_1_name],
                         fnss.get_application_names(self.topo, 10))
        self.assertEqual(self.app_1_props,
                         fnss.get_application_properties(self.topo, 10,
                                                    self.app_1_name))
        fnss.add_application(self.topo, 10, self.app_1_name, self.app_2_props)
        self.assertEqual([self.app_1_name],
                         fnss.get_application_names(self.topo, 10))
        self.assertEqual(self.app_2_props,
                         fnss.get_application_properties(self.topo, 10,
                                                    self.app_1_name))
        fnss.add_application(self.topo, 10, self.app_2_name, self.app_2_props)
        self.assertEqual(set([self.app_1_name, self.app_2_name]),
                         set(fnss.get_application_names(self.topo, 10)))
        self.assertEqual(self.app_2_props,
                         fnss.get_application_properties(self.topo, 10,
                                                    self.app_1_name))
        self.assertEqual(self.app_2_props,
                         fnss.get_application_properties(self.topo, 10,
                                                    self.app_2_name))
        fnss.remove_application(self.topo, 10, self.app_2_name)
        self.assertEqual([self.app_1_name],
                         fnss.get_application_names(self.topo, 10))
        fnss.remove_application(self.topo, 10, self.app_1_name)
        self.assertEqual([], fnss.get_application_names(self.topo, 10))
        fnss.add_application(self.topo, 10, self.app_1_name, self.app_1_props)
        fnss.add_application(self.topo, 10, self.app_2_name, self.app_1_props)
        fnss.remove_application(self.topo, 10)
        self.assertEqual([], fnss.get_application_names(self.topo, 10))

    def test_clear_applications(self):
        for v in self.topo.nodes():
            fnss.add_application(self.topo, v,
                                 self.app_1_name, self.app_1_props)
            fnss.add_application(self.topo, v,
                                 self.app_2_name, self.app_2_props)
        fnss.clear_applications(self.topo)
        for v in self.topo.nodes():
            self.assertEqual([], fnss.get_application_names(self.topo, v))



"""Measurement units used by FNSS"""

__all__ = [
    'capacity_units',
    'time_units',
    'distance_units',
    'convert_capacity_value',
    'convert_time_value'
          ]


# Average Earth radius, in Km
EARTH_RADIUS = 6371

distance_units = {'m': 0.001, 'km': 1, 'Km': 1}

# http://en.wikipedia.org/wiki/Data_rate_units
capacity_units = {'Tbps': 10**12,        'Gbps': 10**9,      'Mbps': 10**6,      
                  'Kbps': 1000,          'kbps': 1000,       'bps': 1,
                  'Tbit/s': 10**12,      'Gbit/s': 10**9,    'Mbit/s': 10**6,    
                  'Kbit/s': 1000,        'kbit/s': 1000,     'bit/s': 1,
                  'Tb': 10**12,          'Gb': 10**9,        'Mb': 10**6,        
                  'Kb': 1000,            'kb': 1000,         'b': 1,
                  'TBps': 8*(10**12),    'GBps': 8*(10**9),  'MBps': 8*(10**6),  
                  'KBps': 8000,          'kBps': 8000,       'Bps': 8,
                  'TB/s': 8*(10**12),    'GB/s': 8*(10**9),  'MB/s': 8*(10**6),  
                  'KB/s': 8000,          'kB/s': 8000,       'B/s': 8,
                  'TB': 8*(10**12),      'GB': 8*(10**9),    'MB': 8*(10**6),    
                  'KB': 8000,            'kB': 8000,         'B': 8
                  }


time_units = {'minutes': 60*10**3,      'minute': 60*10**3,
              'min': 60*10**3,          'm': 60*10**3,
              'seconds': 10**3,         'second': 10**3,
              'sec': 10**3,             's': 10**3,
              'milliseconds': 1,        'millisecond': 1,
              'millisec': 1,            'ms': 1,
              'microseconds': 0.001,    'microsecond': 0.001,
              'microsec': 0.001,        'us': 0.001,
              'nanoseconds': 0.000001,  'nanosecond': 0.000001,
              'nanosec': 0.000001,      'ns': 0.000001
              }


def convert_capacity_value(value, old_unit, new_unit):
    """Convert a capacity value from a unit to another one.

    Parameters
    ----------
    value : float
        Value of a measure to convert
    old_unit : str
        Unit from which the conversion is made
    new_unit : str
        Unit to which the conversion is made

    Returns
    -------
    converted_value : float
        Capacity value in the new unit
    """
    return value * capacity_units[old_unit] / capacity_units[new_unit]

def convert_time_value(value, old_unit, new_unit):
    """Convert a time/delay value from a unit to another one.

    Parameters
    ----------
    value : float
        Value of a measure to convert
    old_unit : str
        Unit from which the conversion is made
    new_unit : str
        Unit to which the conversion is made

    Returns
    -------
    converted_value : float
        Time/delay value in the new unit
    """
    return value * time_units[old_unit] / time_units[new_unit]



import unittest

import fnss
from fnss.util import package_available

class Test(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    @unittest.skipUnless(package_available('mininet'), 'Requires Mininet')
    def test_to_mininet(self):
        t = fnss.Topology()
        t.add_path([1, 2, 3, 4])
        for n in (1, 4):
            t.node[n]['type'] = 'host'
        for n in (2, 3):
            t.node[n]['type'] = 'switch'
        fnss.set_capacities_constant(t, 10, 'Mbps')
        fnss.set_delays_constant(t, 10, 'ms')
        mn_topo = fnss.to_mininet(t, relabel_nodes=False)
        self.assertIsNotNone(mn_topo)
        hosts = mn_topo.hosts()
        switches = mn_topo.switches()
        for h in '1', '4':
            self.assertIn(h, hosts)
        for s in '2', '3':
            self.assertIn(s, switches)
        mn_topo = fnss.to_mininet(t, relabel_nodes=True)
        # Relabeling should be:
        # 1 -> h1
        # 2 -> s1
        # 3 -> s2
        # 4 -> h2
        self.assertIsNotNone(mn_topo)
        hosts = mn_topo.hosts()
        switches = mn_topo.switches()
        for h in 'h1', 'h2':
            self.assertIn(h, hosts)
        for s in 's1', 's2':
            self.assertIn(s, switches)

    @unittest.skipUnless(package_available('mininet'), 'Requires Mininet')
    def test_from_mininet(self):
        try:
            from mininet.topo import Topo
        except ImportError:
            raise ImportError('You must have Mininet installed to run this test case')
        t = Topo()
        t.addHost("h1")
        t.addHost("h4")
        t.addSwitch("s2")
        t.addSwitch("s3")
        t.addLink("h1", "s2")
        t.addLink("s2", "s3")
        t.addLink("s3", "h4")
        fnss_topo = fnss.from_mininet(t)
        self.assertIsNotNone(fnss_topo)
        for h in "h1", "h4":
            self.assertEqual(fnss_topo.node[h]['type'], 'host')
        for s in "s2", "s3":
            self.assertEqual(fnss_topo.node[s]['type'], 'switch')



"""Adapter for jFed

Provides function to convert an FNSS Topology object into a jFed
rspec file and viceversa.

`jFed <http://jfed.iminds.be/>_` is a Java-based framework to support the
integration of federated testbed, developed by
`iMinds <http://www.iminds.be/>_` in the contex of the
`Fed4FIRE <http://www.fed4fire.eu/>_` project funded by the Framework
Programme 7 (FP7) of the European Union.
"""
from __future__ import division
import xml.etree.cElementTree as ET

import networkx as nx

from fnss import Topology, get_delays, get_capacities
import fnss.util as util
import fnss.units as units


__all__ = ['to_jfed', 'from_jfed']


def to_jfed(topology, path, testbed="wall1.ilabt.iminds.be", encoding="utf-8", prettyprint=True):
    """Convert a topology object into an RSPEC file for jFed

    Parameters
    ----------
    topology : Topology
        The topology object
    path : str
        The file to which the RSPEC will be written
    testbed : str, optional
        URI of the testbed to use
    encoding : str, optional
        The encoding of the target file
    prettyprint : bool, optional
        Indent the XML code in the output file

    Notes
    -----
    It currently supports only undirected topologies, if a topology is directed
    it is converted to undirected
    """
    if topology.is_directed():
        topology = topology.to_undirected()
    topology = nx.convert_node_labels_to_integers(topology)

    if 'capacity_unit' in topology.graph:
        capacity_norm = units.capacity_units[topology.graph['capacity_unit']] / units.capacity_units['Kbps']
    if 'delay_unit' in topology.graph:
        delay_norm = units.time_units[topology.graph['delay_unit']] / units.time_units['ms']
    delays = get_delays(topology)
    capacities = get_capacities(topology)
    # Node positions (randomly generated)
    pos = nx.random_layout(topology)
    # Create mapping between links and interface IDs
    if_names = {}
    for v in topology.adj:
        next_hops = sorted(topology.adj[v].keys())
        if_names[v] = {next_hop: i for i, next_hop in enumerate(next_hops)}
    head = ET.Element('rspec')
    head.attrib["generated_by"] = "FNSS"
    head.attrib['xsi:schemaLocation'] = "http://www.geni.net/resources/rspec/3 http://www.geni.net/resources/rspec/3/request.xsd"
    head.attrib['xmlns'] = "http://www.geni.net/resources/rspec/3"
    head.attrib["xmlns:jFed"] = "http://jfed.iminds.be/rspec/ext/jfed/1"
    head.attrib["xmlns:jFedBonfire"] = "http://jfed.iminds.be/rspec/ext/jfed-bonfire/1"
    head.attrib["xmlns:delay"] = "http://www.protogeni.net/resources/rspec/ext/delay/1"
    head.attrib["xmlns:xsi"] = "http://www.w3.org/2001/XMLSchema-instance"
    # Iterate over nodes
    for v in topology.nodes():
        node = ET.SubElement(head, 'node')
        node.attrib['client_id'] = "node%s" % str(v)
        node.attrib['component_manager_id'] = "urn:publicid:IDN+%s+authority+cm" % testbed
        node.attrib["exclusive"] = "true"
        sliver_type = ET.SubElement(node, 'sliver_type')
        sliver_type.attrib['name'] = topology.node[v]['sliver_type'] if 'sliver_type' in topology.node[v] else 'raw-pc'
        location = ET.SubElement(node, 'jFed:location')
        x, y = pos[v]
        location.attrib['x'] = str(1000 * x)
        location.attrib['y'] = str(500 * y)
        for if_name in if_names[v].values():
            interface = ET.SubElement(node, 'interface')
            interface.attrib['client_id'] = "node%s:if%s" % (str(v), str(if_name))
    # The convention in jFed is to identify links with "linkX" where X is an
    # integer but making sure that links and nodes have different integers
    link_id = topology.number_of_nodes() - 1
    for u, v in topology.edges():
        link_id += 1
        link = ET.SubElement(head, 'link')
        link.attrib['client_id'] = "link%s" % str(link_id)
        component_manager = ET.SubElement(link, 'component_manager')
        component_manager.attrib['name'] = "urn:publicid:IDN+%s+authority+cm" % testbed
        u_if = "node%s:if%s" % (str(u), str(if_names[u][v]))
        v_if = "node%s:if%s" % (str(v), str(if_names[v][u]))
        for source, dest in ((u_if, v_if), (v_if, u_if)):
            prop = ET.SubElement(link, 'property')
            prop.attrib["source_id"] = source
            prop.attrib["dest_id"] = dest
            if (u, v) in delays:
                prop.attrib['latency'] = str(delay_norm * delays[(u, v)])
            if (u, v) in capacities:
                prop.attrib['capacity'] = str(capacity_norm * capacities[(u, v)])
            interface_ref = ET.SubElement(link, 'interface_ref')
            interface_ref.attrib['client_id'] = source
    if prettyprint:
        util.xml_indent(head)
    ET.ElementTree(head).write(path, encoding=encoding)


def from_jfed(path):
    """Read a jFed RSPEC file and returns an FNSS topology modelling the
    network topology of the jFed experiment specification.

    Parameters
    ----------
    path : str
        The path of the jFed RSPEC file to parse

    Returns
    -------
    topology: Topology
        The parsed topology

    Notes
    -----
    This function does not support directed topologies and unidirectional links

    It is possible in jFed to create multipoint links (links with more than 2
    endpoints). Such types of link cannot be modelled in FNSS. Therefore, any
    attempt to convert an RSPEC with such links will fail.
    """
    # This implementation could be improved by using SOAP libraries, but it
    # would require to add another dependency to the project.
    # The current implementation, although not really elegant, works fine
    tree = ET.parse(path)
    head = tree.getroot()
    xmlns = "http://www.geni.net/resources/rspec/3"
    topology = Topology()
    # Flags to indicate whether the parsed topology is annotated with
    # capacities and delays
    has_delays = False
    has_capacities = False
    # Dict mapping interface name to the node it belongs to
    if_map = {}
    # Iterate over nodes
    for node in head.findall('{%s}node' % xmlns):
        client_id = node.attrib['client_id']
        component_manager_id = node.attrib['component_manager_id']
        exclusive = bool(node.attrib['exclusive'])
        for interface in node.findall('{%s}interface' % xmlns):
            if_id = interface.attrib['client_id']
            if_map[if_id] = client_id
        topology.add_node(client_id,
                          component_manager_id=component_manager_id,
                          exclusive=exclusive)
    # Iterate over links
    for edge in head.findall('{%s}link' % xmlns):
        client_id = edge.attrib['client_id']
        component_manager = edge.find('{%s}component_manager' % xmlns).attrib['name']
        interfaces = edge.findall('{%s}interface_ref' % xmlns)
        # A link may connect more than two nodes. These links cannot be
        # represented in an FNSS Topology.
        # This statement also encompasses the potential case of one interface
        # only that should not happen if the file is correctly formatted.
        if len(interfaces) != 2:
            raise ValueError("Link %s is not a point-to-point link but a shared "
                             "medium connecting %d interfaces. These links are "
                             "not supported. Change your topology and try again"
                             % (client_id, len(interfaces)))
        u = if_map[interfaces[0].attrib['client_id']]
        v = if_map[interfaces[1].attrib['client_id']]
        edge_attr = dict(component_manager=component_manager,
                         client_id=client_id)
        # There are normally two property tags per edge. We only parse one
        # because should have same attributes if links are bidirectional.
        # This function does not support unidirectional links
        prop = edge.find('{%s}property' % xmlns)
        if prop:
            if 'capacity' in prop.attrib:
                edge_attr['capacity'] = prop.attrib['capacity']
                has_capacities = True
            if 'latency' in prop.attrib:
                edge_attr['delay'] = prop.attrib['latency']
                has_delays = True
        topology.add_edge(u, v, **edge_attr)
    # Set capacity and delay units
    if has_capacities:
        topology.graph['capacity_unit'] = 'kbps'
    if has_delays:
        topology.graph['delay_unit'] = 'ms'
    return topology



"""Functions to parse topologies from datasets or from other generators."""
import re
import math

import networkx as nx

from fnss.topologies.topology import Topology, DirectedTopology
from fnss.util import geographical_distance


__all__ = [
    'parse_rocketfuel_isp_map',
    'parse_rocketfuel_isp_latency',
    'parse_caida_as_relationships',
    'parse_inet',
    'parse_abilene',
    'parse_brite',
    'parse_topology_zoo',
    'parse_ashiip'
          ]


# Parser for RocketFuel ISP (router-level) maps .cch files
def parse_rocketfuel_isp_map(path):
    """
    Parse a network topology from RocketFuel ISP map file.

    The ASes provided by the RocketFuel dataset are the following:

    +------+---------------------+-------+--------+------------+------------+
    | ASN  | Name                | Span  | Region | Nodes (r1) | Nodes (r0) |
    +======+=====================+=======+========+============+============+
    | 1221 | Telstra (Australia) | world | AUS    |  2999      |  378 (318) |
    | 1239 | Sprintlink (US)     | world | US     |  8352      |  700 (604) |
    | 1755 | EBONE (Europe)      | world | Europe |   609      |  172       |
    | 2914 | Verio (US)          | world | US     |  7109      | 1013       |
    | 3257 | Tiscali (Europe)    | world | Europe |   855      |  248 (240) |
    | 3356 | Level 3 (US)        | world | US     |  3447      |  652       |
    | 3967 | Exodus (US)         | world | US     |   917      |  215 (201) |
    | 4755 | VSNL (India)        | world | India  |   121      |   12       |
    | 6461 | Abovenet (US)       | world | US     |     0      |  202       |
    | 7018 | AT&T (US)           | world | US     | 10152      |  656 (631) |
    +------+---------------------+-------+--------+------------+------------+

    Parameters
    ----------
    path : str
        The path of the file containing the RocketFuel map. It should have
        extension .cch

    Returns
    -------
    topology : DirectedTopology
        The object containing the parsed topology.

    Notes
    -----
    The returned topology is always directed. If an undirected topology is
    desired, convert it using the DirectedTopology.to_undirected() method.

    Each node of the returned graph has the following attributes:
     * **type**: string
     * **location**: string (optional)
     * **address**: string
     * **r**: int
     * **backbone**: boolean (optional)

    Each edge of the returned graph has the following attributes:
     * type : string, which can either be *internal* or *external*

    If the topology contains self-loops (links starting and ending in the same
    node) they are stripped from the topology.

    Raises
    ------
    ValueError
        If the provided file cannot be parsed correctly.

    Examples
    --------
    >>> import fnss
    >>> topology = fnss.parse_rocketfuel_isp_map('1221.r0.cch')
    """
    topology = DirectedTopology(type='rocket_fuel')
    comment_char = '#'

    with  open(path, "r") as f:
        for line in f.readlines():
            if comment_char in line:
                # split on comment char, keep only the part before
                line, _ = line.split(comment_char, 1)
                line = line.strip()
            if len(line) == 0:
                continue
            # Parse line.
            if line.startswith("-"):
                # Case external node
                # -euid =externaladdress rn
                try:
                    node = int(re.findall("-\d+", line)[0])
                    address = (re.findall("=\S+", line)[0])[1:]  # .strip("=")
                    r = int(re.findall("r\d$", line)[0][1:])  # .strip("r"))
                except IndexError:
                    raise ValueError('Invalid input file. Parsing failed '\
                                     'while trying to parse an external node')
                topology.add_node(node, type='external', address=address, r=r)
            else:
                # Case internal node
                # uid @loc [+] [bb] (num_neigh) [&ext] -> <nuid-1> <nuid-2>
                # ... {-euid} ... =name[!] rn
                try:
                    node = int(re.findall("\d+", line)[0])
                    node_location = re.findall("@\S*", line)[0]
                    node_location = re.sub("[\+@]", "", node_location)
                    r = int(re.findall("r\d$", line)[0][1:])  # .strip("r"))
                    address = (re.findall("=\S+", line)[0])[1:]  # .strip("=")
                except IndexError:
                    raise ValueError('Invalid input file. Parsing failed '\
                                     'while trying to parse an internal node')
                internal_links = re.findall("<(\d+)>", line)
                external_links = re.findall("{(-?\d+)}", line)
                backbone = (len(re.findall("\sbb\s", line)) > 0)
                topology.add_node(node, type='internal',
                                  location=node_location,
                                  address=address, r=r, backbone=backbone)
                for link in internal_links:
                    link = int(link)
                    if node != link:
                        topology.add_edge(node, link, type='internal')
                for link in external_links:
                    link = int(link)
                    if node != link:
                        topology.add_edge(node, link, type='external')
    return topology


def parse_rocketfuel_isp_latency(latencies_path, weights_path=None):
    """
    Parse a network topology from RocketFuel ISP topology file (latency.intra)
    with inferred link latencies and optionally annotate the topology with
    inferred weights (weights.infra).

    The ASes provided by the RocketFuel dataset are the following:

    +------+---------------------+-------+--------+-------+-------------------+
    | ASN  | Name                | Span  | Region | Nodes | Lrgst conn. comp. |
    +======+=====================+=======+========+=======+===================+
    | 1221 | Telstra (Australia) | world | AUS    |  108  |        104        |
    | 1239 | Sprintlink (US)     | world | US     |  315  |        315        |
    | 1755 | EBONE (Europe)      | world | Europe |   87  |         87        |
    | 3257 | Tiscali (Europe)    | world | Europe |  161  |        161        |
    | 3967 | Exodus (US)         | world | US     |   79  |         79        |
    | 6461 | Abovenet (US)       | world | US     |  141  |        138        |
    +------+---------------------+-------+--------+-------+-------------------+

    Parameters
    ----------
    latencies_path : str
        The path of the file containing the RocketFuel latencies file.
        It should have extension .intra
    weights_path : str, optional
        The path of the file containing the RocketFuel weights file.
        It should have extension .intra

    Returns
    -------
    topology : DirectedTopology
        The object containing the parsed topology.

    Notes
    -----
    The returned topology is directed. It can be converted using the
    DirectedTopology.to_undirected() method if an undirected topology
    is desired.

    Each node of the returned graph has the following attributes:
     * **name**: string
     * **location**: string

    Each edge of the returned graph has the following attributes:
     * **delay** : int
     * **wdights** : float (only if a weights file was specified)

    Raises:
    -------
    ValueError
        If the provided file cannot be parsed correctly

    Examples
    --------
    >>> import fnss
    >>> topology = fnss.parse_rocketfuel_isp_latency('1221.latencies.intra')
    """
    topology = DirectedTopology(type='rocket_fuel', delay_unit='ms')
    comment_char = '#'
    node_dict = dict()
    node_count = 0

    with open(latencies_path, "r") as f:
        for line in f.readlines():
            if comment_char in line:
                # split on comment char, keep only the part before
                line, _ = line.split(comment_char, 1)
                line = line.strip()
            if len(line) == 0:
                continue
            u_str, v_str, delay = line.split()
            try:
                # Edges endpoints and delay are separated by a space.
                # An edge endpoint generally has the format <location>,<router-name>
                # but there are some endpoints which don't, e.g. London4083 in
                # topology 1239. This function tries first to parse by splitting
                # at the comma. If it fails, then separates number from location.
                # If this also fails, it just keeps the node name as it is.
                try:
                    u_location, u_name = u_str.split(',')
                except ValueError:
                    match = re.match(r"([a-z]+)([0-9]+)", u_str, re.I)
                    if match:
                        u_location, u_name = match.groups()
                    else:
                        u_location = u_name = u_str

                try:
                    v_location, v_name = v_str.split(',')
                except ValueError:
                    match = re.match(r"([a-z]+)([0-9]+)", v_str, re.I)
                    if match:
                        v_location, v_name = match.groups()
                    else:
                        v_location = v_name = v_str

                if delay.isdigit():
                    delay = int(delay)
                else:
                    raise ValueError('Invalid delay value: %s' % delay)
            except ValueError:
                raise ValueError('Invalid latencies file. Parsing failed '\
                                 'while trying to parse an edge')

            if u_str not in node_dict:
                node_dict[u_str] = node_count
                topology.add_node(node_count, location=u_location, name=u_name)
                node_count += 1
            if v_str not in node_dict:
                node_dict[v_str] = node_count
                topology.add_node(node_count, location=v_location, name=v_name)
                node_count += 1

            u = node_dict[u_str]
            v = node_dict[v_str]
            topology.add_edge(u, v, delay=delay)

    if weights_path:
        with open(weights_path, "r") as f:
            for line in f.readlines():
                if comment_char in line:
                    # split on comment char, keep only the part before
                    line, _ = line.split(comment_char, 1)
                    line = line.strip()
                if len(line) == 0:
                    continue
                try:
                    u_str, v_str, weight = line.split()
                except ValueError:
                    raise ValueError('Invalid weight file. Parsing failed '\
                                     'while trying to parse an edge')
                try:
                    weight = float(weight)
                except ValueError:
                    raise ValueError('Invalid weight value: %s' % weight)
                try:
                    u = node_dict[u_str]
                    v = node_dict[v_str]
                except KeyError:
                    raise ValueError("The weight file includes edge (%s, %s), "
                                     "which was not included in the latencies file"
                                     % (u_str, v_str))
                topology.adj[u][v]['weight'] = weight
    return topology


def parse_caida_as_relationships(path):
    """
    Parse a topology from the CAIDA AS relationships dataset

    Parameters
    ----------
    path : str
        The path to the CAIDA AS relationships file

    Returns
    -------
    topology : DirectedTopology

    Notes
    -----
    The node names of the returned topology are the the ASN of the of the AS
    they represent and edges are annotated with the relationship between ASes
    they connect. The relationship values can either be *customer*, *peer* or
    *sibling*.

    References
    ----------
    http://www.caida.org/data/active/as-relationships/
    http://as-rank.caida.org/data/
    """
    topology = DirectedTopology(type='caida_as_relationships')
    comment_char = '#'
    relationships_dict = {-1: 'customer', 0: 'peer', 2: 'sibling'}

    with open(path, "r") as f:
        for line in f.readlines():
            if comment_char in line:
                # split on comment char, keep only the part before
                line, _ = line.split(comment_char, 1)
                line = line.strip()
            if len(line) == 0:
                continue
            entry = line.split('|')
            try:
                from_as = int(entry[0])
                to_as = int(entry[1])
                relationship = relationships_dict[int(entry[2])]
            except (ValueError, IndexError, KeyError):
                raise ValueError('Invalid input file. Parsing failed while trying'\
                                 ' to parse a line')
            if from_as != to_as:
                topology.add_edge(from_as, to_as, type=relationship)
    return topology


def parse_inet(path):
    """
    Parse a topology from an output file generated by the Inet topology
    generator

    Parameters
    ----------
    path : str
        The path to the Inet output file

    Returns
    -------
    topology : Topology

    Notes
    -----
    Each node of the returned topology object is labeled with *latitude* and
    *longitude* attributes. These attributes are not expressed in degrees but
    in Kilometers.
    """
    topology = Topology(type='inet', distance_unit='Km')
    with open(path, "r") as f:
        lines = f.readlines()
    sep = re.compile('[\s\t]')
    first_line = sep.split(lines[0].strip())
    try:
        n_nodes = int(first_line[0])
        n_links = int(first_line[1])
    except (ValueError, IndexError):
        raise ValueError('Invalid input file. '\
                         'Cannot parse the number of nodes and links')
    if len(lines) != 1 + n_nodes + n_links:
        raise ValueError('Invalid input file. '\
                         'It does not have as many lines as expected')
    i = 0
    for line in lines[1:]:
        entry = sep.split(line.strip())
        if i < n_nodes:
            i += 1
            try:
                node_id = int(entry[0])
                longitude = int(entry[1])
                latitude = int(entry[2])
            except (ValueError, IndexError):
                raise ValueError('Invalid input file. Parsing failed while '\
                                 'trying to parse a node')
            topology.add_node(node_id, latitude=latitude, longitude=longitude)
        else:
            try:
                u = int(entry[0])
                v = int(entry[1])
                weight = int(entry[2])
                x_u = topology.node[u]['longitude']
                y_u = topology.node[u]['latitude']
                x_v = topology.node[v]['longitude']
                y_v = topology.node[v]['latitude']
                length = float(math.sqrt((x_v - x_u) ** 2 + (y_v - y_u) ** 2))
            except (ValueError, IndexError):
                raise ValueError('Invalid input file. Parsing failed while '\
                                 'trying to parse a link')
            topology.add_edge(u, v, weight=weight, length=length)
    return topology


# Ignore external links
# Node parameters: city, latitude, longitude
# Link parameters: capacity, weight[, link_index, link_type]
def parse_abilene(topology_path, links_path=None):
    """
    Parse the Abilene topology.

    Parameters
    ----------
    topology_path : str
        The path of the Abilene topology file
    links_path : str, optional
        The path of the Abilene links file

    Returns
    -------
    topology : DirectedTopology
    """
    topology = DirectedTopology(type='abilene',
                         capacity_unit='kbps',
                         distance_unit='Km')
    comment_char = '#'
    link_type_dict = {0: 'internal', 1: 'inbound', 2: 'outbound'}
    line_type = None
    with open(topology_path, "r") as f:
        for line in f.readlines():
            if comment_char in line:
                # split on comment char, keep only the part before
                line, _ = line.split(comment_char, 1)
            line = line.strip()
            if len(line) > 0:
                if line == 'router' or line == 'link':
                    line_type = line
                    continue
                if line_type == 'router':
                    node_entry = line.split('\t')
                    try:
                        name = node_entry[0]
                        city = node_entry[1]
                        latitude = float(node_entry[2])
                        longitude = float(node_entry[3])
                    except (ValueError, IndexError):
                        raise ValueError('Invalid input file. Parsing failed '\
                                         'while trying to parse a router')
                    topology.add_node(name, city=city, latitude=latitude,
                               longitude=longitude)
                elif line_type == 'link':
                    sep = re.compile('[\s\t]')
                    link_entry = sep.split(line)
                    try:
                        u = link_entry[0]
                        v = link_entry[1]
                        capacity = int(link_entry[2])
                        lon_u = topology.node[u]['longitude']
                        lat_u = topology.node[u]['latitude']
                        lon_v = topology.node[v]['longitude']
                        lat_v = topology.node[v]['latitude']
                        length = geographical_distance(lat_v, lon_v, lat_u, lon_u)
                        weight = int(link_entry[3])
                    except (ValueError, IndexError):
                        raise ValueError('Invalid input file. Parsing failed '\
                                         'while trying to parse a link')
                    topology.add_edge(u, v, capacity=capacity,
                               weight=weight, length=length)
                else:
                    raise ValueError('Invalid input file. Found a line that '\
                                     'I cannot interpret')
    if links_path:
        with  open(links_path, "r") as f:
            for line in f.readlines():
                if comment_char in line:
                    # split on comment char, keep only the part before
                    line, _ = line.split(comment_char, 1)
                line = line.strip()
                if len(line) > 0:
                    sep = re.compile('[\s\t]')
                    link_entry = sep.split(line)
                    try:
                        u, v = link_entry[0].split(',', 1)
                        if u == '*' or v == '*':  # ignore external links
                            continue
                        link_index = int(link_entry[1])
                        link_type = link_type_dict[int(link_entry[2])]
                    except (ValueError, IndexError):
                        raise ValueError('Invalid input file. '\
                                         'Parsing failed while trying to '\
                                         'parse a link from links_file')
                    topology.adj[u][v]['link_index'] = link_index
                    topology.adj[u][v]['link_type'] = link_type
    return topology


def parse_brite(path, capacity_unit='Mbps', delay_unit='ms',
                distance_unit='Km', directed=True):
    """
    Parse a topology from an output file generated by the BRITE topology
    generator

    Parameters
    ----------
    path : str
        The path to the BRITE output file
    capacity_unit : str, optional
        The unit in which link capacity values are expresses in the BRITE file
    delay_unit : str, optional
        The unit in which link delay values are expresses in the BRITE file
    distance_unit : str, optional
        The unit in which node coordinates are expresses in the BRITE file
    directed : bool, optional
        If True, the topology is parsed as directed topology.

    Returns
    -------
    topology : Topology or DirectedTopology

    Notes
    -----
    Each node of the returned topology object is labeled with *latitude* and
    *longitude* attributes. These attributes are not expressed in degrees but
    in *distance_unit*.
    """
    # BRITE output format:
    # http://www.cs.bu.edu/brite/user_manual/node29.html
    topology = DirectedTopology() if directed else Topology()
    topology.graph = {'type':'brite', 'capacity_unit':capacity_unit,
               'delay_unit':delay_unit, 'distance_unit': distance_unit}
    line_type = None
    with open(path, "r") as f:
        for line in f.readlines():
            if line.startswith('Nodes:'):
                line_type = 'node'
            elif line.startswith('Edges:'):
                line_type = 'edge'
            elif line[0].isdigit():
                elements = line.strip().split("\t")
                if line_type == 'node':
                    # Parse node
                    try:
                        node_id = int(elements[0])
                        longitude = float(elements[1])
                        latitude = float(elements[2])
                        # indegree = int(elements[3])
                        # outdegree = int(elements[4])
                        as_id = int(elements[5])
                        # Node type can be:
                        # AS-only: AS_NODE
                        # Router-only: RT_NODE
                        # Top-down: RT_NODE, RT_BORDER
                        # Bottom-up: RT_NODE
                        node_type = elements[6]
                    except (ValueError, IndexError):
                        raise ValueError('Invalid input file. Parsing failed '\
                                         'while trying to parse a node')
                    topology.add_node(node_id, latitude=latitude,
                                      longitude=longitude, type=node_type)
                    if as_id > 0:
                        topology.node[node_id]['AS'] = as_id
                elif line_type == 'edge':
                    # Parse link
                    try:
                        edge_id = int(elements[0])
                        from_node = int(elements[1])
                        to_node = int(elements[2])
                        length = float(elements[3])
                        delay = float(elements[4])
                        capacity = float(elements[5])
                        # from_as = elements[6]
                        # to_as = elements[7]
                        # Link type can be:
                        # AS-only: E_AS
                        # Router-only: E_RT
                        # Top-down: E_AS, E_RT
                        # bottom-up: E_RT
                        link_type = elements[8]
                    except (ValueError, IndexError):
                        raise ValueError('Invalid input file. Parsing failed '\
                                         'while trying to parse a link')
                    topology.add_edge(from_node, to_node, id=edge_id,
                                      length=length, delay=delay,
                                      capacity=capacity, type=link_type)
                else:
                    continue
    return topology


def parse_topology_zoo(path):
    """
    Parse a topology from the Topology Zoo dataset.

    Parameters
    ----------
    path : str
        The path to the Topology Zoo file

    Returns
    -------
    topology : Topology or DirectedTopology
        The parsed topology.

    Notes
    -----
    If the parsed topology contains bundled links, i.e. multiple links between
    the same pair or nodes, the topology is parsed correctly but each bundle of
    links is represented as a single link whose capacity is the sum of the
    capacities of the links of the bundle (if capacity values were provided).
    The returned topology has a boolean attribute named *link_bundling* which
    is True if the topology contains at list one bundled link or False
    otherwise. If the topology contains bundled links, then each link has an
    additional boolean attribute named *bundle* which is True if that specific
    link was bundled in the original topology or False otherwise.
    """
    def try_convert_int(value):
        """
        Try to convert a string to an int. If not possible, returns the given
        value unchanged
        """
        if type(value) != int:
            try:
                value = int(value)
            except ValueError:
                pass
        return value
    if path.endswith('.gml'):
        topo_zoo_graph = nx.read_gml(path)
    elif path.endswith('.graphml'):
        topo_zoo_graph = nx.read_graphml(path)
    else:
        raise ValueError('Invalid input file format. It must either be a GML '\
                         'or GraphML file (with extensions .gml or .graphml)')
    topology = DirectedTopology() if topo_zoo_graph.is_directed() \
               else Topology()
    topology.graph['type'] = 'topology_zoo'
    topology.graph['distance_unit'] = 'Km'
    topology.graph['link_bundling'] = topo_zoo_graph.is_multigraph()
    for tv in topo_zoo_graph.nodes():
        v = try_convert_int(tv)
        topology.add_node(v)
        if 'label' in topo_zoo_graph.node[tv]:
            topology.node[v]['label'] = topo_zoo_graph.node[tv]['label']
        try:
            longitude = topo_zoo_graph.node[tv]['Longitude']
            latitude = topo_zoo_graph.node[tv]['Latitude']
            topology.node[v]['longitude'] = longitude
            topology.node[v]['latitude'] = latitude
        except KeyError:
            pass
    for tv, tu in topo_zoo_graph.edges():
        v = try_convert_int(tv)
        u = try_convert_int(tu)
        if u == v:
            continue
        topology.add_edge(v, u)
        if 'Latitude' in topo_zoo_graph.node[tv] and \
                'Longitude' in topo_zoo_graph.node[tv] and \
                'Latitude' in topo_zoo_graph.node[tu] and \
                'Longitude' in topo_zoo_graph.node[tu]:
            lat_v = topo_zoo_graph.node[tv]['Latitude']
            lon_v = topo_zoo_graph.node[tv]['Longitude']
            lat_u = topo_zoo_graph.node[tu]['Latitude']
            lon_u = topo_zoo_graph.node[tu]['Longitude']
            length = geographical_distance(lat_v, lon_v, lat_u, lon_u)
            topology.adj[v][u]['length'] = length
        if topo_zoo_graph.is_multigraph():
            edge = topo_zoo_graph.adj[tv][tu]
            topology.adj[v][u]['bundle'] = True if len(edge) > 1 else False
            capacity = 0
            for edge_attr in list(edge.values()):
                if 'LinkSpeedRaw' in edge_attr:
                    capacity += edge_attr['LinkSpeedRaw']
            if capacity > 0:
                topology.adj[v][u]['capacity'] = capacity
        else:
            if 'LinkSpeedRaw' in topo_zoo_graph.adj[tv][tu]:
                topology.adj[v][u]['capacity'] = \
                        topo_zoo_graph.adj[tv][tu]['LinkSpeedRaw']
    if len(nx.get_edge_attributes(topology, 'capacity')) > 0:
        topology.graph['capacity_unit'] = 'bps'
    return topology


def parse_ashiip(path):
    """
    Parse a topology from an output file generated by the aShiip topology
    generator

    Parameters
    ----------
    path : str
        The path to the aShiip output file

    Returns
    -------
    topology : Topology
    """
    topology = Topology(type='ashiip')
    with open(path, "r") as f:
        for line in f.readlines():
            # There is no documented aShiip format but we assume that if the line
            # does not start with a number it is not part of the topology
            if line[0].isdigit():
                node_ids = re.findall("\d+", line)
                if len(node_ids) < 3:
                    raise ValueError('Invalid input file. Parsing failed while '\
                                     'trying to parse a line')
                node = int(node_ids[0])
                level = int(node_ids[1])
                topology.add_node(node, level=level)
                for i in range(2, len(node_ids)):
                    topology.add_edge(node, int(node_ids[i]))
    return topology



from os import environ, path
import unittest

import fnss

RES_DIR = environ['test.res.dir'] if 'test.res.dir' in environ else None
TMP_DIR = environ['test.tmp.dir'] if 'test.tmp.dir' in environ else None

class Test(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    @unittest.skipIf(TMP_DIR is None, "Temp folder not present")
    def test_to_from(self):
        f = path.join(TMP_DIR, 'jfed-tofrom.rspec')
        t_in = fnss.Topology()
        t_in.add_path([1, 2, 3, 4])
        fnss.to_jfed(t_in, f)
        t_out = fnss.from_jfed(f)
        self.assertEqual(t_in.number_of_nodes(), t_out.number_of_nodes())
        self.assertEqual(t_in.number_of_edges(), t_out.number_of_edges())
        self.assertEqual(set(dict(t_in.degree()).values()),
                         set(dict(t_out.degree()).values()))

    @unittest.skipIf(TMP_DIR is None, "Temp folder not present")
    def test_to_jfed(self):
        t = fnss.Topology()
        t.add_path([1, 2, 3, 4])
        fnss.to_jfed(t, path.join(TMP_DIR, 'jfed-to.rspec'))
        pass

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_from_jfed(self):
        rspec = path.join(RES_DIR, 'jfed-success.rspec')
        t = fnss.from_jfed(rspec)
        self.assertTrue(len(t) > 0)



"""Generate canonical deterministic topologies"""
import networkx as nx
from fnss.topologies.topology import Topology, DirectedTopology


__all__ = [
    'ring_topology',
    'line_topology',
    'star_topology',
    'full_mesh_topology',
    'k_ary_tree_topology',
    'dumbbell_topology',
    'chord_topology',
           ]


def ring_topology(n):
    """
    Return a ring topology of n nodes

    Parameters
    ----------
    n : int
        The number of nodes

    Returns
    -------
    topology : A Topology object
    """
    if not isinstance(n, int):
        raise TypeError('n argument must be of int type')
    if n < 1:
        raise ValueError('n argument must be a positive integer')
    G = Topology(nx.path_graph(n))
    G.add_edge(n - 1, 0)
    G.name = "ring_topology(%d)" % (n)
    G.graph['type'] = 'ring'
    return G


def line_topology(n):
    """
    Return a line topology of n nodes

    Parameters
    ----------
    n : int
        The number of nodes

    Returns
    -------
    topology : A Topology object

    """
    if not isinstance(n, int):
        raise TypeError('n argument must be of int type')
    if n < 1:
        raise ValueError('n argument must be a positive integer')
    G = Topology(nx.path_graph(n))
    G.name = "line_topology(%d)" % (n)
    G.graph['type'] = 'line'
    return G


def star_topology(n):
    """
    Return a star (a.k.a hub-and-spoke) topology of :math:`n+1` nodes

    The root (hub) node has id 0 while all leaf (spoke) nodes have id
    :math:`(1, n+1)`.

    Each node has the attribute type which can either be *root* (for node 0) or
    *leaf* for all other nodes

    Parameters
    ----------
    n : int
        The number of leaf nodes

    Returns
    -------
    topology : A Topology object
    """
    if not isinstance(n, int):
        raise TypeError('n argument must be of int type')
    if n < 1:
        raise ValueError('n argument must be a positive integer')
    G = Topology(nx.star_graph(n))
    G.name = "star_topology(%d)" % (n)
    G.graph['type'] = 'star'
    G.node[0]['type'] = 'root'
    for v in range(1, n + 1):
        G.node[v]['type'] = 'leaf'
    return G


def full_mesh_topology(n):
    """
    Return a fully connected mesh topology of n nodes

    Parameters
    ----------
    n : int
        The number of nodes

    Returns
    -------
    topology : A Topology object
    """
    if not isinstance(n, int):
        raise TypeError('n argument must be of int type')
    if n < 1:
        raise ValueError('n argument must be a positive integer')
    G = Topology(nx.complete_graph(n))
    G.name = "full_mesh_topology(%d)" % (n)
    G.graph['type'] = 'full_mesh'
    return G


def k_ary_tree_topology(k, h):
    """
    Return a balanced k-ary tree topology of with depth h

    Each node has two attributes:
     * type: which can either be *root*, *intermediate* or *leaf*
     * depth:math:`(0, h)` the height of the node in the tree, where 0 is the
       root and h are leaves.

    Parameters
    ----------
    k : int
        The branching factor of the tree
    h : int
        The height or depth of the tree

    Returns
    -------
    topology : A Topology object
    """
    if not isinstance(k, int) or not isinstance(h, int):
        raise TypeError('k and h arguments must be of int type')
    if k <= 1:
        raise ValueError("Invalid k parameter. It should be > 1")
    if h < 1:
        raise ValueError("Invalid h parameter. It should be >=1")
    G = Topology(nx.balanced_tree(k, h))
    G.name = "k_ary_tree_topology(%d,%d)" % (k, h)
    G.graph['type'] = 'tree'
    G.graph['k'] = k
    G.graph['h'] = h
    G.node[0]['type'] = 'root'
    G.node[0]['depth'] = 0
    # Iterate through the tree to assign labels to nodes
    v = 1
    for depth in range(1, h + 1):
        for _ in range(k ** depth):
            G.node[v]['depth'] = depth
            G.node[v]['type'] = 'leaf'  if (depth == h) else 'intermediate'
            v += 1
    return G


def dumbbell_topology(m1, m2):
    """
    Return a dumbbell topology consisting of two star topologies
    connected by a path.

    More precisely, two star graphs :math:`K_{m1}` form the left and right
    bells, and are connected by a path :math:`P_{m2}`.

    The :math:`2*m1+m2`  nodes are numbered as follows.
     * :math:`0,...,m1-1` for the left barbell,
     * :math:`m1,...,m1+m2-1` for the path,
     * :math:`m1+m2,...,2*m1+m2-1` for the right barbell.

    The 3 subgraphs are joined via the edges :math:`(m1-1,m1)` and
    :math:`(m1+m2-1,m1+m2)`. If m2 = 0, this is merely two star topologies
    joined together.

    Please notice that this dumbbell topology is different from the barbell
    graph generated by networkx's barbell_graph function. That barbell graph
    consists of two complete graphs connected by a path. This consists of two
    stars whose roots are connected by a path. This dumbbell topology is
    particularly useful for simulating transport layer protocols.

    All nodes and edges of this topology have an attribute *type* which can be
    either *right bell*, *core* or *left_bell*

    Parameters
    ----------
    m1 : int
        The number of nodes in each bell
    m2 : int
        The number of nodes in the path

    Returns
    -------
    topology : A Topology object
    """
    if not isinstance(m1, int) or not isinstance(m2, int):
        raise TypeError('m1 and m2 arguments must be of int type')
    if m1 < 2:
        raise ValueError("Invalid graph description, m1 should be >= 2")
    if m2 < 1:
        raise ValueError("Invalid graph description, m2 should be >= 1")

    G = Topology(type='dumbbell')
    G.name = "dumbbell_topology(%d,%d)" % (m1, m2)

    # left bell
    G.add_node(m1)
    for v in range(m1):
        G.add_node(v, type='left_bell')
        G.add_edge(v, m1, type='left_bell')

    # right bell
    for v in range(m1):
        G.add_node(v + m1 + m2, type='right_bell')
        G.add_edge(v + m1 + m2, m1 + m2 - 1, type='right_bell')

    # connecting path
    for v in range(m1, m1 + m2 - 1):
        G.node[v]['type'] = 'core'
        G.add_edge(v, v + 1, type='core')
    G.node[m1 + m2 - 1]['type'] = 'core'

    return G


def chord_topology(m, r=1):
    """Return a Chord topology, as described in [1]_:

    Chord is a Distributed Hash Table (DHT) providing guaranteed correctness.
    In Chord, both nodes and keys are identified by sequences of :math:`m`
    bits. Keys can be resolved in at most :math:`log(n)` steps (with :math:`n`
    being the number of nodes) as long as each node maintains a routing table
    o :math:`n` entries.

    In this implementation, it is possible only to generate topologies with a
    number of nodes :math:`n = 2^m`. where :math:`m` is the length (in bits) of
    the keys used by Chord and also corresponds the the size of the finger
    table kept by each node.

    The :math:`r` argument is the number of nearest successors which can be
    optionally kept at each node to guarantee correctness in case of node
    failures.

    Parameters
    ----------
    m : int
        The length of keys (in bits), which also corresponds to the length of
        the finger table of each node
    r : int, optional
        The length of the nearest successors table

    Returns
    -------
    G : DirectedTopology
        A Chord topology

    References
    ----------
    .. [1] I. Stoica, R. Morris, D. Karger, M. Frans Kaashoek, H. Balakrishnan,
           Chord: A Scalable Peer-to-peer Lookup Service for Internet
           Applications. Proceedings of the ACM SIGCOMM 2001 conference on Data
           communication (SIGCOMM '09). ACM, New York, NY, USA.
    """
    if not isinstance(m, int) or not isinstance(r, int):
        raise TypeError("m and r must be integers")
    if m < 2:
        raise ValueError("m must be an integer >= 2")
    if  r < 1 or r > 2 ** m - 1:
        raise ValueError("r must be an integer and 1 <= r <= 2^m")
    # n is the number of nodes
    n = 2 ** m
    G = DirectedTopology()
    for v in range(n):
        for u in range(m):
            G.add_edge(v, (v + 2 ** u) % n)
    if r > 2:
        for v in range(n):
            for u in range(v + 3, v + r + 1):
                G.add_edge(v, u % n)
    return G




"""
Rocketfuel topology and traffic matrix
======================================

This example shows how to import a topology from RocketFuel, configure it
(assign capacities, weights and delays), generate a traffic matrix and
save topology and traffic matrix to XML files.
"""
import fnss
import random

# Import RocketFuel topology
# Replace the filename with the actual location of the file you want to parse
topology = fnss.parse_rocketfuel_isp_map("rocket-fuel-topo-file.cch")

# add capacities
capacities = [1, 10, 40]
capacity_unit = 'Gbps'
fnss.set_capacities_edge_betweenness(topology, capacities, capacity_unit,
                                weighted=False)

# add weights proportional to inverse of capacity
fnss.set_weights_inverse_capacity(topology)

# add constant link delays of 2 ms
fnss.set_delays_constant(topology, 2, delay_unit='ms')

# generate cyclostationary traffic matrix (period 7 days, 24 samples per day)
tmc = fnss.sin_cyclostationary_traffic_matrix(
       topology,
       mean=0.5,  # average flow in TM is 0,5 Gbps
       stddev=0.05,  # this is the std among average flows of different OD pairs
       gamma=0.8,  # gamma and log_psi are parameters for fitting the std of
       log_psi=-0.33,  # volume fluctuations over time. Look at Nucci et al. paper
       delta=0.2,  # traffic variation from period max and avg as fraction of average
       n=24,  # number of samples per each period
       periods=7,  # number of periods
       max_u=0.9,  # max link utilization desired
       origin_nodes=None,  # Specify origin and destination nodes. If None,
       destination_nodes=None  # all nodes of the topology are both
       )  # origin and destination nodes of traffic


# now we generate a static traffic matrix, but this time instead of generating
# a matrix where all nodes are both origin and destinations, we pick up only
# few nodes as sources and destinations of traffic.
# Let's select 5 sources and 5 destinations
nodes = topology.nodes()
origins = random.sample(nodes, 5)
destinations = random.sample(nodes, 5)
# generate traffic matrix
tms = fnss.static_traffic_matrix(topology, mean=0.5, stddev=0.05, max_u=0.9,
                                 origin_nodes=origins,
                                 destination_nodes=destinations)

# save topology on a file
fnss.write_topology(topology, 'topology.xml')

# save traffic matrices on files
fnss.write_traffic_matrix(tmc, 'cyclostationary-traffic-matrix.xml')
fnss.write_traffic_matrix(tms, 'static-traffic-matrix.xml')



from os import environ, path
from math import exp
import unittest

from numpy import isinf

import fnss

TMP_DIR = environ['test.tmp.dir'] if 'test.tmp.dir' in environ else None


class Test(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        # set up topology used for all traffic matrix tests
        cls.G = fnss.glp_topology(n=30, m=1, m0=10, p=0.2, beta=-2, seed=1)
        fnss.set_capacities_random(cls.G, {10: 0.5, 20: 0.3, 40: 0.2}, 
                              capacity_unit='Mbps')

    @classmethod
    def tearDownClass(cls):
        pass
    
    def setUp(self):
        pass


    def tearDown(self):
        pass

    def test_traffic_matrix_class(self):
        tm = fnss.TrafficMatrix(volume_unit='Mbps')
        tm.add_flow(1, 2, 1000)
        tm.add_flow(1, 3, 1500)
        tm.add_flow(3, 'Four', 4000)
        tm[(4, 5)] = 3000
        tm[('Orig', 'Dest')] = 3000
        self.assertEqual(tm[(1, 3)], 1500)
        self.assertEqual(tm.flow[1][3], 1500)
        self.assertEqual(tm.flows()[(1, 3)], 1500)
        self.assertEqual(5, len(tm))
        flow = tm.pop_flow(1, 2)
        self.assertEqual(1000, flow)
        del tm[(1, 3)]
        self.assertEqual(3, len(tm))
        self.assertFalse(1 in tm.flow)
        self.assertFalse((1, 3) in tm)

    def test_traffic_matrix_sequence_class(self):
        tms = fnss.TrafficMatrixSequence()
        tm1 = fnss.TrafficMatrix(volume_unit='Mbps')
        tm1.add_flow(1, 2, 1000)
        tm1.add_flow(1, 3, 1500)
        tm1.add_flow(3, 4, 4000)
        tm2 = fnss.TrafficMatrix(volume_unit='Mbps')
        tm2.add_flow(1, 2, 2000)
        tm2.add_flow(1, 3, 3500)
        tm2.add_flow(3, 4, 5000)
        tm3 = fnss.TrafficMatrix(volume_unit='Mbps')
        tm3.add_flow(1, 2, 5000)
        tm3.add_flow(1, 3, 6500)
        tm3.add_flow(3, 4, 7000)
        tms.append(tm1)
        tms.append(tm2)
        tms.append(tm3)
        self.assertEqual(1000, tms[0][(1, 2)])
        self.assertEqual(1000, tms[0].flow[1][2])
        self.assertEqual(1000, tms.get(0)[(1, 2)])
        self.assertEqual(1000, tms.get(0).flows()[(1, 2)])
        self.assertEqual(3, len(tms))
        del tms[0]
        self.assertEqual(2, len(tms))
        
    def test_link_loads(self):
        topo = fnss.ring_topology(5)
        fnss.set_capacities_constant(topo, 100, capacity_unit='Mbps')
        tm = fnss.TrafficMatrix(volume_unit='Mbps')
        tm.add_flow(0, 1, 20)
        tm.add_flow(1, 0, 30)
        tm.add_flow(0, 2, 40)
        tm.add_flow(1, 4, 70)
        tm.add_flow(4, 2, 50)
        load = fnss.link_loads(topo, tm)
        self.assertAlmostEqual(0.6, load[(0, 1)])
        self.assertAlmostEqual(1.0, load[(1, 0)])
        self.assertAlmostEqual(0.4, load[(1, 2)])
        self.assertAlmostEqual(0.0, load[(2, 1)])
        self.assertAlmostEqual(0.7, load[(0, 4)])
        self.assertAlmostEqual(0.5, load[(4, 3)])

    def test_link_loads_ecmp(self):
        topo = fnss.ring_topology(5)
        topo.add_edge(1, 3)
        fnss.set_capacities_constant(topo, 100, capacity_unit='Mbps')
        tm = fnss.TrafficMatrix(volume_unit='Mbps')
        tm.add_flow(0, 1, 20)
        tm.add_flow(1, 0, 30)
        tm.add_flow(0, 2, 40)
        tm.add_flow(1, 4, 70)
        rm = {0 :
                { 0: [[0]],
                  1: [[0, 1]],
                  2: [[0, 1, 2]],
                  3: [[0, 1, 3], [0, 4, 3]],
                  4: [[0, 4]]
                },
              1 :
                { 0: [[1, 0]],
                  1: [[1]],
                  2: [[1, 2]],
                  3: [[1, 3]],
                  4: [[1, 3, 4], [1, 0, 4]]
                },
              2 :
                { 0: [[2, 1, 0]],
                  1: [[2, 1]],
                  2: [[2]],
                  3: [[2, 3]],
                  4: [[2, 3, 4]]
                },
              3 :
                { 0: [[3, 1, 0], [3, 4, 0]],
                  1: [[3, 1]],
                  2: [[3, 2]],
                  3: [[3]],
                  4: [[3, 4]]
                },
              4 :
                { 0: [[4, 0]],
                  1: [[4, 0, 1], [4, 3, 1]],
                  2: [[4, 3, 2]],
                  3: [[4, 3]],
                  4: [[4]]
                }}

        load = fnss.link_loads(topo, tm, routing_matrix=rm, ecmp=True)
        self.assertAlmostEqual(0.6, load[(0, 1)])
        self.assertAlmostEqual(0.65, load[(1, 0)])
        self.assertAlmostEqual(0.4, load[(1, 2)])
        self.assertAlmostEqual(0.0, load[(2, 1)])
        self.assertAlmostEqual(0.35, load[(0, 4)])
        self.assertAlmostEqual(0.35, load[(3, 4)])
        self.assertAlmostEqual(0.0, load[(4, 3)])
        self.assertAlmostEqual(0.0, load[(4, 0)])
        self.assertAlmostEqual(0.0, load[(2, 1)])
        self.assertAlmostEqual(0.0, load[(2, 3)])
        
    def test_static_traffic_matrix(self):
        tm = fnss.static_traffic_matrix(self.G, 10, 8, max_u=0.9)
        self.assertAlmostEqual(0.9, max(fnss.link_loads(self.G, tm).values()))
        self.assertLessEqual(0, min(fnss.link_loads(self.G, tm).values()))

    def test_static_traffic_matrix_partial_od_pairs(self):
        origin_nodes = [1, 2, 3]
        destination_nodes = [3, 4, 5]
        od_pairs = [(o, d) for o in origin_nodes for d in destination_nodes if o != d]
        tm = fnss.static_traffic_matrix(self.G, 10, 8,
                                        origin_nodes=origin_nodes,
                                        destination_nodes=destination_nodes, 
                                        max_u=0.9)
        tm_od_pairs = tm.od_pairs()
        self.assertEqual(len(od_pairs), len(tm_od_pairs))
        for od in od_pairs:
            self.assertTrue(od in tm_od_pairs)
        self.assertAlmostEqual(0.9, max(fnss.link_loads(self.G, tm).values()))
        self.assertLessEqual(0, min(fnss.link_loads(self.G, tm).values()))

    def test_stationary_traffic_matrix(self):
        tms = fnss.stationary_traffic_matrix(self.G, mean=10, stddev=3.5, gamma=5, 
                                             log_psi=-0.3, n=5, max_u=0.9)
        self.assertEqual(5, len(tms))
        self.assertAlmostEqual(0.9, max([max(fnss.link_loads(self.G, tm).values()) for tm in tms]))
        self.assertLessEqual(0, min([min(fnss.link_loads(self.G, tm).values()) for tm in tms]))


    def test_sin_cyclostationary_traffic_matrix(self):
        tms = fnss.sin_cyclostationary_traffic_matrix(self.G, 10, 0.2, gamma=0.3, 
                                                      log_psi=-0.3, delta=0.2, 
                                                      n=24, periods=2, max_u=0.9)
        self.assertEqual(48, len(tms))
        self.assertAlmostEqual(0.9, max([max(fnss.link_loads(self.G, tm).values()) for tm in tms]))
        self.assertLessEqual(0, min([min(fnss.link_loads(self.G, tm).values()) for tm in tms]))
    
    def test_sin_cyclostationary_traffic_matrix_low_log_psi(self):
        # Test that with very low value of log_psi and/or gamma to test 
        # that FNSS deals properly with division by 0 cases
        log_psi = [-1500, -200]
        for lp in log_psi:
            if exp(lp) == 0:
                self.assertRaises(ValueError, 
                                  fnss.sin_cyclostationary_traffic_matrix,
                                  self.G, mean=10, stddev=0.2, 
                                  gamma=0.3, log_psi=lp, delta=0.2, 
                                  n=24, periods=2, max_u=0.9)
            else:
                try:
                    tms = fnss.sin_cyclostationary_traffic_matrix(
                                      self.G, mean=10, stddev=0.2, 
                                      gamma=0.3, log_psi=lp, delta=0.2, 
                                      n=24, periods=2, max_u=0.9)
                except ValueError:
                    pass
                else:
                    volumes = [tm[y] for tm in tms for y in tm.flows()]
                    valid_volumes = [vol >= 0 and not isinf(vol) for vol in volumes]
                    self.assertTrue(all(valid_volumes))

    @unittest.skipIf(TMP_DIR is None, "Temp folder not present")
    def test_read_write_tm(self):
        tm = fnss.static_traffic_matrix(self.G, mean=10, stddev=0.1, max_u=0.9)
        tmp_tm_file = path.join(TMP_DIR, 'tms.xml')
        fnss.write_traffic_matrix(tm, tmp_tm_file)
        read_tm = fnss.read_traffic_matrix(tmp_tm_file)
        u, v = tm.od_pairs()[2]
        self.assertAlmostEqual(tm[(u, v)], read_tm[(u, v)])
    
    @unittest.skipIf(TMP_DIR is None, "Temp folder not present")
    def test_read_write_tms(self):
        tms = fnss.stationary_traffic_matrix(self.G, mean=10, stddev=0.1, gamma=1.2, 
                                        log_psi=-0.3, n=5, max_u=0.9)
        tmp_tms_file = path.join(TMP_DIR, 'tms.xml')
        fnss.write_traffic_matrix(tms, tmp_tms_file)
        read_tms = fnss.read_traffic_matrix(tmp_tms_file)
        u, v = tms[3].od_pairs()[2]
        self.assertAlmostEqual(tms[3][(u, v)], read_tms[3][(u, v)])

    def test_validate_traffic_matrix(self):
        topology = fnss.DirectedTopology()
        topology.add_path([1, 2, 3])
        topology.add_path([3, 2, 1])
        topology.add_edge(3, 4)
        fnss.set_capacities_constant(topology, 1, 'Mbps')
        flows_valid_load = {1: {3: 0.4}, 2: {4: 0.3}}
        flows_invalid_load = {1: {3: 0.4}, 2: {4: 0.7}}
        flows_invalid_routes = {4: {1: 0.4}}
        flows_invalid_pairs = {5: {2: 0.1}}
        self.assertTrue(fnss.validate_traffic_matrix(topology, fnss.TrafficMatrix('Mbps', flows_valid_load), validate_load=False))
        self.assertTrue(fnss.validate_traffic_matrix(topology, fnss.TrafficMatrix('Mbps', flows_valid_load), validate_load=True))
        self.assertTrue(fnss.validate_traffic_matrix(topology, fnss.TrafficMatrix('Mbps', flows_invalid_load), validate_load=False))
        self.assertFalse(fnss.validate_traffic_matrix(topology, fnss.TrafficMatrix('Mbps', flows_invalid_load), validate_load=True))
        self.assertFalse(fnss.validate_traffic_matrix(topology, fnss.TrafficMatrix('Mbps', flows_invalid_routes), validate_load=False))
        self.assertFalse(fnss.validate_traffic_matrix(topology, fnss.TrafficMatrix('Mbps', flows_invalid_routes), validate_load=True))
        self.assertFalse(fnss.validate_traffic_matrix(topology, fnss.TrafficMatrix('Mbps', flows_invalid_pairs), validate_load=False))
        self.assertFalse(fnss.validate_traffic_matrix(topology, fnss.TrafficMatrix('Mbps', flows_invalid_pairs), validate_load=True))

    def test_validate_traffic_matrix_diff_units(self):
        topo = fnss.line_topology(2)
        fnss.set_capacities_constant(topo, 1, capacity_unit='Gbps')
        valid_tm = fnss.TrafficMatrix(volume_unit='Mbps')
        valid_tm.add_flow(0, 1, 999)
        self.assertTrue(fnss.validate_traffic_matrix(topo, valid_tm, validate_load=True))
        invalid_tm = fnss.TrafficMatrix(volume_unit='Mbps')
        invalid_tm.add_flow(0, 1, 1001)
        self.assertFalse(fnss.validate_traffic_matrix(topo, invalid_tm, validate_load=True))



"""Basic functions and classes for operating on network topologies."""
import xml.etree.cElementTree as ET
import networkx as nx
import fnss.util as util


__all__ = [
    'Topology',
    'DirectedTopology',
    'od_pairs_from_topology',
    'fan_in_out_capacities',
    'rename_edge_attribute',
    'rename_node_attribute',
    'read_topology',
    'write_topology',
           ]


class BaseTopology(object):
    """Base class for generic topology. Provides utility methods for listing
    nodes and edge properties.
    """

    def capacities(self):
        """Return a dictionary of all link capacities, keyed by link

        Returns
        -------
        capacities : dict
            A dictionary of link capacity, keyed by link
        """
        return nx.get_edge_attributes(self, 'capacity')

    def delays(self):
        """
        Return a dictionary of all link delays, keyed by link

        Returns
        -------
        delays : dict
            A dictionary of link delays, keyed by link
        """
        return nx.get_edge_attributes(self, 'delay')

    def weights(self):
        """Return a dictionary of all link weights, keyed by link

        Returns
        -------
        weights : dict
            A dictionary of all link weights, keyed by link
        """
        return nx.get_edge_attributes(self, 'weight')

    def buffers(self):
        """Return a dictionary of all buffer sizes, keyed by interface

        Returns
        -------
        buffers : dict
            A dictionary of buffer sizes, keyed by interface. The interface is
            a tuple (u, v) which is the link to which the the interface is
            outputting
        """
        return nx.get_edge_attributes(self, 'buffer')

    def stacks(self):
        """Return a dictionary of all node stacks, keyed by node

        Returns
        -------
        stacks : dict
            A dictionary of all node stacks, keyed by node. Each node stack is
            a tuple (name, properties) where name is the stack name and
            properties is a the dictionary
        """
        return nx.get_node_attributes(self, 'stack')

    def applications(self):
        """Return a dictionary of all applications deployed, keyed by node

        Returns
        -------
        applications : dict
            A dictionary of all applications deployed, keyed by node.
        """
        return nx.get_node_attributes(self, 'application')


class Topology(nx.Graph, BaseTopology):
    """Base class for undirected topology"""

    def __init__(self, data=None, name="", **kwargs):
        """Initialize the topology

        Parameters
        ----------
        data : input graph
            Data to initialize the topology.  If data=None (default) an empty
            topology is created. The data can be an edge list, or any
            FNSS Topology or NetworkX graph object.  If the corresponding
            optional Python packages are installed the data can also be a NumPy
            matrix or 2d ndarray, a SciPy sparse matrix, or a PyGraphviz graph.
        name : string, optional
            An optional name for the graph. Default is ""
        **kwargs : keyword arguments, optional
            Attributes to add to graph as key=value pairs.
        """
        super(Topology, self).__init__(data, name=name, **kwargs)

    def copy(self):
        """Return a copy of the topology.

        Returns
        -------
        topology : Topology
            A copy of the topology.

        See Also
        --------
        to_directed: return a directed copy of the topology.

        Notes
        -----
        This makes a complete copy of the topology including all of the
        node or edge attributes.

        Examples
        --------
        >>> topo = Topology()
        >>> topo.add_path([0,1,2,3])
        >>> copied_topo = topo.copy()

        """
        return Topology(super(Topology, self).copy())

    def subgraph(self, nbunch):
        """Return the subgraph induced on nodes in nbunch.

        The induced subgraph of the graph contains the nodes in nbunch
        and the edges between those nodes.

        Parameters
        ----------
        nbunch : list, iterable
            A container of nodes which will be iterated through once.

        Returns
        -------
        topology : Topology
            A subgraph of the graph with the same edge attributes.

        Notes
        -----
        The graph, edge or node attributes just point to the original graph.
        So changes to the node or edge structure will not be reflected in
        the original graph while changes to the attributes will.

        To create a subgraph with its own copy of the edge/node attributes use:
        Topology(G.subgraph(nbunch))

        If edge attributes are containers, a deep copy can be obtained using:
        G.subgraph(nbunch).copy()

        For an inplace reduction of a graph to a subgraph you can remove nodes:
        G.remove_nodes_from([ n in G if n not in set(nbunch)])

        Examples
        --------
        >>> topo = Topology()
        >>> topo.add_path([0,1,2,3])
        >>> topo2 = topo.subgraph([0,1,2])
        >>> topo2.edges()
        [(0, 1), (1, 2)]
        """
        return Topology(super(Topology, self).subgraph(nbunch))

    def to_directed(self):
        """Return a directed representation of the topology.

        Returns
        -------
        topology : DirectedTopology
            A directed topology with the same name, same nodes, and with
            each edge (u,v,data) replaced by two directed edges
            (u,v,data) and (v,u,data).

        Notes
        -----
        This returns a 'deepcopy' of the edge, node, and
        graph attributes which attempts to completely copy
        all of the data and references.

        This is in contrast to the similar D=DirectedTopology(G) which returns
        a shallow copy of the data.

        See the Python copy module for more information on shallow
        and deep copies, http://docs.python.org/library/copy.html.

        Examples
        --------
        >>> topo = Topology()
        >>> topo.add_path([0,1])
        >>> topo2 = topo.to_directed()
        >>> topo2.edges()
        [(0, 1), (1, 0)]

        If already directed, return a (deep) copy

        >>> topo = DirectedTopology()
        >>> topo.add_path([0,1])
        >>> topo2 = topo.to_directed()
        >>> topo2.edges()
        [(0, 1)]
        """
        return DirectedTopology(super(Topology, self).to_directed())

    def to_undirected(self):
        """Return an undirected copy of the topology.

        Returns
        -------
        topology : Topology
            A undirected copy of the topology.

        See Also
        --------
        copy, add_edge, add_edges_from

        Notes
        -----
        This returns a 'deepcopy' of the edge, node, and
        graph attributes which attempts to completely copy
        all of the data and references.

        This is in contrast to the similar G=Topology(D) which returns a
        shallow copy of the data.

        See the Python copy module for more information on shallow
        and deep copies, http://docs.python.org/library/copy.html.

        Examples
        --------
        >>> topo = Topology()   # or MultiGraph, etc
        >>> topo.add_path([0,1])
        >>> topo2 = topo.to_directed()
        >>> topo2.edges()
        [(0, 1), (1, 0)]
        >>> topo3 = topo2.to_undirected()
        >>> topo3.edges()
        [(0, 1)]
        """
        return Topology(super(Topology, self).to_undirected())


class DirectedTopology(nx.DiGraph, BaseTopology):
    """Base class for directed topology"""

    def __init__(self, data=None, name="", **kwargs):
        """Initialize the topology

        Parameters
        ----------
        data : input graph
            Data to initialize the topology.  If data=None (default) an empty
            topology is created. The data can be an edge list, or any
            FNSS Topology or NetworkX graph object.  If the corresponding
            optional Python packages are installed the data can also be a NumPy
            matrix or 2d ndarray, a SciPy sparse matrix, or a PyGraphviz graph.
        name : string, optional
            An optional name for the graph. Default is ""
        **kwargs : keyword arguments, optional
            Attributes to add to graph as key=value pairs.
        """
        super(DirectedTopology, self).__init__(data, name=name, **kwargs)

    def copy(self):
        """Return a copy of the topology.

        Returns
        -------
        topology : DirectedTopology
            A copy of the topology.

        See Also
        --------
        to_undirected: return a undirected copy of the topology.

        Notes
        -----
        This makes a complete copy of the topology including all of the
        node or edge attributes.

        Examples
        --------
        >>> topo = DirectedTopology()
        >>> topo.add_path([0,1,2,3])
        >>> copied_topo = topo.copy()
        """
        return DirectedTopology(super(DirectedTopology, self).copy())

    def subgraph(self, nbunch):
        """Return the subgraph induced on nodes in nbunch.

        The induced subgraph of the graph contains the nodes in nbunch
        and the edges between those nodes.

        Parameters
        ----------
        nbunch : list, iterable
            A container of nodes which will be iterated through once.

        Returns
        -------
        topology : DirectedTopology
            A subgraph of the graph with the same edge attributes.

        Notes
        -----
        The graph, edge or node attributes just point to the original graph.
        So changes to the node or edge structure will not be reflected in
        the original graph while changes to the attributes will.

        To create a subgraph with its own copy of the edge/node attributes use:
        Topology(G.subgraph(nbunch))

        If edge attributes are containers, a deep copy can be obtained using:
        G.subgraph(nbunch).copy()

        For an inplace reduction of a graph to a subgraph you can remove nodes:
        G.remove_nodes_from([ n in G if n not in set(nbunch)])

        Examples
        --------
        >>> topo = Topology()
        >>> topo.add_path([0,1,2,3])
        >>> topo2 = topo.subgraph([0,1,2])
        >>> topo2.edges()
        [(0, 1), (1, 2)]
        """
        return DirectedTopology(super(DirectedTopology, self).subgraph(nbunch))

    def to_directed(self):
        """Return a directed representation of the topology.

        Returns
        -------
        topology : DirectedTopology
            A directed topology with the same name, same nodes, and with
            each edge (u,v,data) replaced by two directed edges
            (u,v,data) and (v,u,data).

        Notes
        -----
        This returns a 'deepcopy' of the edge, node, and
        graph attributes which attempts to completely copy
        all of the data and references.

        This is in contrast to the similar D=DirectedTopology(G) which returns
        a shallow copy of the data.

        See the Python copy module for more information on shallow
        and deep copies, http://docs.python.org/library/copy.html.

        Examples
        --------
        >>> topo = Topology()
        >>> topo.add_path([0,1])
        >>> topo2 = topo.to_directed()
        >>> topo2.edges()
        [(0, 1), (1, 0)]

        If already directed, return a (deep) copy

        >>> topo = DirectedTopology()
        >>> topo.add_path([0,1])
        >>> topo2 = topo.to_directed()
        >>> topo2.edges()
        [(0, 1)]
        """
        return DirectedTopology(super(DirectedTopology, self).to_directed())

    def to_undirected(self):
        """Return an undirected copy of the topology.

        Returns
        -------
        topology : Topology
            A undirected copy of the topology.

        See Also
        --------
        copy, add_edge, add_edges_from

        Notes
        -----
        This returns a 'deepcopy' of the edge, node, and
        graph attributes which attempts to completely copy
        all of the data and references.

        This is in contrast to the similar G=Topology(D) which returns a
        shallow copy of the data.

        See the Python copy module for more information on shallow
        and deep copies, http://docs.python.org/library/copy.html.

        Examples
        --------
        >>> topo = Topology()   # or MultiGraph, etc
        >>> topo.add_path([0,1])
        >>> topo2 = topo.to_directed()
        >>> topo2.edges()
        [(0, 1), (1, 0)]
        >>> topo3 = topo2.to_undirected()
        >>> topo3.edges()
        [(0, 1)]
        """
        return Topology(super(DirectedTopology, self).to_undirected())


def od_pairs_from_topology(topology):
    """Calculate all possible origin-destination pairs of the topology.
    This function does not simply calculate all possible pairs of the topology
    nodes. Instead, it only returns pairs of nodes connected by at least
    a path.

    Parameters
    ----------
    topology : Topology or DirectedTopology
        The topology whose OD pairs are calculated

    Returns
    -------
    od_pair : list
        List containing all origin destination tuples.

    Examples
    --------
    >>> import fnss
    >>> topology = fnss.ring_topology(3)
    >>> fnss.od_pairs_from_topology(topology)
    [(0, 1), (0, 2), (1, 0), (1, 2), (2, 0), (2, 1)]
    """
    if topology.is_directed():
        routes = dict(nx.all_pairs_shortest_path_length(topology))
        return [(o, d) for o in routes for d in routes[o] if o != d]
    else:
        conn_comp = nx.connected_components(topology)
        return [(o, d) for G in conn_comp for o in G for d in G if o != d]


def fan_in_out_capacities(topology):
    """Calculate fan-in and fan-out capacities for all nodes of the topology.

    The fan-in capacity of a node is the sum of capacities of all incoming
    links, while the fan-out capacity is the sum of capacities of all outgoing
    links.

    Parameters
    ----------
    topology : Topology
        The topology object whose fan-in and fan-out capacities are calculated.
        This topology must be annotated with link capacities.

    Returns
    -------
    fan_in_out_capacities : tuple (fan_in, fan_out)
        A tuple of two dictionaries, representing, respectively the fan-in and
        fan-out capacities keyed by node.

    Notes
    -----
    This function works correctly for both directed and undirected topologies.
    If the topology is undirected, the returned dictionaries of fan-in and
    fan-out capacities are identical.

    Examples
    --------
    >>> import fnss
    >>> topology = fnss.star_topology(3)
    >>> fnss.set_capacities_constant(topology, 10, 'Mbps')
    >>> in_cap, out_cap = fnss.fan_in_out_capacities(topology)
    >>> in_cap
    {0: 30, 1: 10, 2: 10, 3: 10}
    >>> out_cap
    {0: 30, 1: 10, 2: 10, 3: 10}
    """
    if not topology.is_directed():
        topology = topology.to_directed()
    fan_in = {}
    fan_out = {}
    for node in topology.nodes():
        node_fan_in = 0
        node_fan_out = 0
        for predecessor in topology.predecessors(node):
            node_fan_in += topology.adj[predecessor][node]['capacity']
        for successor in topology.successors(node):
            node_fan_out += topology.adj[node][successor]['capacity']
        fan_in[node] = node_fan_in
        fan_out[node] = node_fan_out
    return fan_in, fan_out


def rename_edge_attribute(topology, old_attr, new_attr):
    """Rename all edges attributes with a specific name to a new name

    Parameters
    ----------
    topology : Topology
        The topology object
    old_attr : any hashable type
        Old attribute name
    new_attr : any hashable type
        New attribute name

    Example
    -------
    >>> import fnss
    >>> topo = fnss.Topology()
    >>> topo.add_edge(1, 2, cost=1)
    >>> topo.add_edge(2, 3, cost=2)
    >>> topo.edges(data=True)
    [(1, 2, {'cost': 1}), (2, 3, {'cost': 2})]
    >>> fnss.rename_edge_attribute(topo, 'cost', 'weight')
    >>> topo.edges(data=True)
    [(1, 2, {'weight': 1}), (2, 3, {'weight': 2})]
    """
    for u, v in topology.edges():
        if old_attr in topology.adj[u][v]:
            topology.adj[u][v][new_attr] = topology.edge[u][v][old_attr]
            del topology.adj[u][v][old_attr]


def rename_node_attribute(topology, old_attr, new_attr):
    """Rename all nodes attributes with a specific name to a new name

    Parameters
    ----------
    topology : Topology
        The topology object
    old_attr : any hashable type
        Old attribute name
    new_attr : any hashable type
        New attribute name

    Example
    -------
    >>> import fnss
    >>> topo = fnss.Topology()
    >>> topo.add_node(1, pos=(0, 0))
    >>> topo.add_node(2, pos=(1, 1))
    >>> topo.nodes(data=True)
    [(1, {'pos': (0, 0)}), (2, {'pos': (1, 1)})]
    >>> fnss.rename_edge_attribute(topo, 'pos', 'coordinates')
    >>> topo.edges(data=True)
    [(1, {'coordinates': (0, 0)}), (2, {'coordinates': (1, 1)})]
    """
    for v in topology.nodes():
        if old_attr in topology.adj[v]:
            topology.node[v][new_attr] = topology.node[v][old_attr]
            del topology.node[v][old_attr]


def read_topology(path, encoding='utf-8'):
    """Read a topology from an XML file and returns either a Topology or a
    DirectedTopology object

    Parameters
    ----------
    path : str
        The path of the topology XML file to parse
    encoding : str, optional
        The encoding of the file

    Returns
    -------
    topology: Topology or DirectedTopology
        The parsed topology
    """
    tree = ET.parse(path)
    head = tree.getroot()
    topology = Topology() if head.attrib['linkdefault'] == 'undirected' \
                   else DirectedTopology()
    for prop in head.findall('property'):
        name = prop.attrib['name']
        value = util.xml_cast_type(prop.attrib['type'], prop.text)
        topology.graph[name] = value
    for node in head.findall('node'):
        v = util.xml_cast_type(node.attrib['id.type'], node.attrib['id'])
        topology.add_node(v)
        for prop in node.findall('property'):
            name = prop.attrib['name']
            value = util.xml_cast_type(prop.attrib['type'], prop.text)
            topology.node[v][name] = value
        if len(node.findall('stack')) > 0:
            if len(node.findall('stack')) > 1:
                raise ET.ParseError('Invalid topology. ' \
                                    'A node has more than one stack.')
            stack = node.findall('stack')[0]
            stack_name = util.xml_cast_type(stack.attrib['name.type'],
                                        stack.attrib['name'])
            stack_props = {}
            for prop in stack.findall('property'):
                name = prop.attrib['name']
                value = util.xml_cast_type(prop.attrib['type'], prop.text)
                stack_props[name] = value
            topology.node[v]['stack'] = (stack_name, stack_props)
        if len(node.findall('application')) > 0:
            topology.node[v]['application'] = {}
            for application in node.findall('application'):
                app_name = util.xml_cast_type(application.attrib['name.type'],
                                          application.attrib['name'])
                app_props = {}
                for prop in application.findall('property'):
                    name = prop.attrib['name']
                    value = util.xml_cast_type(prop.attrib['type'], prop.text)
                    app_props[name] = value
                topology.node[v]['application'][app_name] = app_props
    for edge in head.findall('link'):
        u = util.xml_cast_type(edge.find('from').attrib['type'],
                           edge.find('from').text)
        v = util.xml_cast_type(edge.find('to').attrib['type'],
                           edge.find('to').text)
        topology.add_edge(u, v)
        for prop in edge.findall('property'):
            name = prop.attrib['name']
            value = util.xml_cast_type(prop.attrib['type'], prop.text)
            topology.adj[u][v][name] = value
    return topology


def write_topology(topology, path, encoding='utf-8', prettyprint=True):
    """Write a topology object on an XML file

    Parameters
    ----------
    topology : Topology
        The topology object to write
    path : str
        The file ob which the topology will be written
    encoding : str, optional
        The encoding of the target file
    prettyprint : bool, optional
        Indent the XML code in the output file
    """
    head = ET.Element('topology')
    head.attrib['linkdefault'] = 'directed' if topology.is_directed() \
                                            else 'undirected'
    for name, value in topology.graph.items():
        prop = ET.SubElement(head, 'property')
        prop.attrib['name'] = name
        prop.attrib['type'] = util.xml_type(value)
        prop.text = str(value)
    for v in topology.nodes():
        node = ET.SubElement(head, 'node')
        node.attrib['id'] = str(v)
        node.attrib['id.type'] = util.xml_type(v)
        for name, value in topology.node[v].items():
            if name is 'stack':
                stack_name, stack_props = topology.node[v]['stack']
                stack = ET.SubElement(node, 'stack')
                stack.attrib['name'] = stack_name
                stack.attrib['name.type'] = util.xml_type(stack_name)
                for prop_name, prop_value in stack_props.items():
                    prop = ET.SubElement(stack, 'property')
                    prop.attrib['name'] = prop_name
                    prop.attrib['type'] = util.xml_type(prop_value)
                    prop.text = str(prop_value)
            elif name is 'application':
                for application_name, application_props in \
                            topology.node[v]['application'].items():
                    application = ET.SubElement(node, 'application')
                    application.attrib['name'] = application_name
                    application.attrib['name.type'] = \
                            util.xml_type(application_name)
                    for prop_name, prop_value in application_props.items():
                        prop = ET.SubElement(application, 'property')
                        prop.attrib['name'] = prop_name
                        prop.attrib['type'] = util.xml_type(prop_value)
                        prop.text = str(prop_value)
            else:
                prop = ET.SubElement(node, 'property')
                prop.attrib['name'] = name
                prop.attrib['type'] = util.xml_type(value)
                prop.text = str(value)
    for u, v in topology.edges():
        link = ET.SubElement(head, 'link')
        from_node = ET.SubElement(link, 'from')
        from_node.attrib['type'] = util.xml_type(u)
        from_node.text = str(u)
        to_node = ET.SubElement(link, 'to')
        to_node.attrib['type'] = util.xml_type(v)
        to_node.text = str(v)
        for name, value in topology.adj[u][v].items():
            prop = ET.SubElement(link, 'property')
            prop.attrib['name'] = name
            prop.attrib['type'] = util.xml_type(value)
            prop.text = str(value)
    if prettyprint:
        util.xml_indent(head)
    ET.ElementTree(head).write(path, encoding=encoding)



from os import environ, path
from re import findall
import unittest

import fnss

RES_DIR = environ['test.res.dir'] if 'test.res.dir' in environ else None

class Test(unittest.TestCase):

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_abilene(self):
        abilene_topo_file = path.join(RES_DIR, 'abilene-topo.txt')
        abilene_links_file = path.join(RES_DIR, 'abilene-links.txt')
        topology = fnss.parse_abilene(abilene_topo_file)
        self.assertEquals(12, topology.number_of_nodes())
        self.assertEquals(30, topology.number_of_edges())
        topology = fnss.parse_abilene(abilene_topo_file, abilene_links_file)
        self.assertEquals(12, topology.number_of_nodes())
        self.assertEquals(30, topology.number_of_edges())
        self.assertTrue(all('link_index' in topology.adj[u][v]
                            and 'link_type' in topology.adj[u][v])
                        for u, v in topology.edges())
        self.assertTrue(all(topology.adj[u][v]['length'] >= 0
                            for u, v in topology.edges()))

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_rockefuel_isp_map(self):
        rocketfuel_file = path.join(RES_DIR, 'rocketfuel-2914.cch')
        topology = fnss.parse_rocketfuel_isp_map(rocketfuel_file)
        self.assertEquals(10961, topology.number_of_nodes())
        self.assertEquals(26070, topology.number_of_edges())

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_rocketfuel_isp_latency(self):
        rocketfuel_file = path.join(RES_DIR, 'rocketfuel-1221.latencies.intra')
        topology = fnss.parse_rocketfuel_isp_latency(rocketfuel_file)
        self.assertEquals(108, topology.number_of_nodes())
        self.assertEquals(306, topology.number_of_edges())
        for _, _, data in topology.edges(data=True):
            self.assertTrue('delay' in data)
            self.assertIsInstance(data['delay'], int)
            self.assertGreaterEqual(data['delay'], 0)

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_rocketfuel_isp_latency_with_weights(self):
        latencies_file = path.join(RES_DIR, 'rocketfuel-1221.latencies.intra')
        weights_file = path.join(RES_DIR, 'rocketfuel-1221.weights.intra')
        topology = fnss.parse_rocketfuel_isp_latency(latencies_file, weights_file)
        self.assertEquals(108, topology.number_of_nodes())
        self.assertEquals(306, topology.number_of_edges())
        for _, _, data in topology.edges(data=True):
            self.assertTrue('delay' in data)
            self.assertTrue('weight' in data)
            self.assertIsInstance(data['delay'], int)
            self.assertIsInstance(data['weight'], float)
            self.assertGreaterEqual(data['delay'], 0)
            self.assertGreater(data['weight'], 0)

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_rocketfuel_isp_latency_overseas_nodes(self):
        rocketfuel_file = path.join(RES_DIR, 'rocketfuel-1239.latencies.intra')
        topology = fnss.parse_rocketfuel_isp_latency(rocketfuel_file)
        self.assertEquals(315, topology.number_of_nodes())
        self.assertEquals(1944, topology.number_of_edges())
        for _, _, data in topology.edges(data=True):
            self.assertTrue('delay' in data)
            self.assertIsInstance(data['delay'], int)
            self.assertGreaterEqual(data['delay'], 0)

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_rocketfuel_isp_latency_with_weights_overseas_nodes(self):
        latencies_file = path.join(RES_DIR, 'rocketfuel-1239.latencies.intra')
        weights_file = path.join(RES_DIR, 'rocketfuel-1239.weights.intra')
        topology = fnss.parse_rocketfuel_isp_latency(latencies_file, weights_file)
        self.assertEquals(315, topology.number_of_nodes())
        self.assertEquals(1944, topology.number_of_edges())
        for _, _, data in topology.edges(data=True):
            self.assertTrue('delay' in data)
            self.assertTrue('weight' in data)
            self.assertIsInstance(data['delay'], int)
            self.assertIsInstance(data['weight'], float)
            self.assertGreaterEqual(data['delay'], 0)
            self.assertGreater(data['weight'], 0)

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_ashiip(self):
        ashiip_file = path.join(RES_DIR, 'ashiip.txt')
        topology = fnss.parse_ashiip(ashiip_file)
        with open(ashiip_file, "r") as f:
            for line in f.readlines():
                if line.startswith(' Size :'):
                    size = int(findall('\d+', line)[0])
                    break
        print("Expected number of nodes: ", size)
        print("Actual number of nodes: ", topology.number_of_nodes())
        self.assertEqual(size, topology.number_of_nodes())
        self.assertEqual(3, topology.degree(57))

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_caida_as_relationships(self):
        caida_file = path.join(RES_DIR, 'caida-as-rel.txt')
        topology = fnss.parse_caida_as_relationships(caida_file)
        self.assertEqual(41203, topology.number_of_nodes())
        self.assertEqual(121309, topology.number_of_edges())
        self.assertEqual('customer', topology.adj[263053][28163]['type'])

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_inet(self):
        inet_file = path.join(RES_DIR, 'inet.txt')
        topology = fnss.parse_inet(inet_file)
        self.assertEqual(3500, topology.number_of_nodes())
        self.assertEqual(6146, topology.number_of_edges())

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_topology_zoo(self):
        topozoo_file = path.join(RES_DIR, 'topozoo-arnes.graphml')
        topology = fnss.parse_topology_zoo(topozoo_file)
        self.assertEqual(type(topology), fnss.Topology)
        self.assertFalse(topology.is_multigraph())
        self.assertEqual(34, topology.number_of_nodes())
        self.assertEqual(46, topology.number_of_edges())
        self.assertEqual(1000000000.0, topology.adj[4][15]['capacity'])
        self.assertEquals('bps', topology.graph['capacity_unit'])
        self.assertTrue(all(topology.adj[u][v]['length'] >= 0
                    for u, v in topology.edges()
                    if 'length' in topology.adj[u][v]))

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_topology_zoo_multigraph(self):
        topozoo_file = path.join(RES_DIR, 'topozoo-garr.graphml')
        topology = fnss.parse_topology_zoo(topozoo_file)
        self.assertEqual(type(topology), fnss.Topology)
        self.assertFalse(topology.is_multigraph())
        self.assertTrue(topology.graph['link_bundling'])
        self.assertEqual(61, topology.number_of_nodes())
        self.assertEqual(75, topology.number_of_edges())
        self.assertEquals('bps', topology.graph['capacity_unit'])
        self.assertEquals(2000000000, topology.adj[37][58]['capacity'])
        bundled_links = [(43, 18), (49, 32), (41, 18), (4, 7),
                          (6, 55), (9, 58), (58, 37), (10, 55),
                         (14, 57), (14, 35), (18, 41), (18, 43),
                         (31, 33), (31, 34), (32, 49), (37, 58)]
        for u, v in topology.edges():
            print(u, v)
            self.assertEquals((u, v) in bundled_links,
                              topology.adj[u][v]['bundle'])

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_topology_zoo_multigraph_directed_topology(self):
        topozoo_file = path.join(RES_DIR, 'topozoo-kdl.graphml')
        topology = fnss.parse_topology_zoo(topozoo_file)
        self.assertEqual(type(topology), fnss.DirectedTopology)
        self.assertFalse(topology.is_multigraph())
        self.assertTrue(topology.graph['link_bundling'])

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_brite_as(self):
        brite_file = path.join(RES_DIR, 'brite-as.brite')
        topology = fnss.parse_brite(brite_file, directed=False)
        self.assertEqual(type(topology), fnss.Topology)
        self.assertEqual(1000, topology.number_of_nodes())
        self.assertEqual(2000, topology.number_of_edges())
        # 851    570    980    2    2    851    AS_NODE
        self.assertTrue(851 in topology.nodes())
        self.assertEqual(570, topology.node[851]['longitude'])
        self.assertEqual(980, topology.node[851]['latitude'])
        self.assertEqual('AS_NODE', topology.node[851]['type'])
        # 1478    716    230    212.11553455605272    0.7075412636166207    0.0011145252848059164    716    230    E_AS    U
        self.assertEquals(1478, topology.adj[716][230]['id'])
        self.assertAlmostEquals(212.11553455605272,
                                topology.adj[716][230]['length'], 0.01)

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_brite_router(self):
        pass

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_brite_bottomup(self):
        pass

    @unittest.skipIf(RES_DIR is None, "Resources folder not present")
    def test_parse_brite_topdown(self):
        pass



"""Functions to deploy and configure protocol stacks and applications
on network nodes
"""

__all__ = [
    'add_stack',
    'get_stack',
    'remove_stack',
    'clear_stacks',
    'add_application',
    'get_application_names',
    'get_application_properties',
    'remove_application',
    'clear_applications',
           ]


def add_stack(topology, node, name, properties=None, **kwargs):
    """Set stack on a node.

    If the node already has a stack, it is overwritten

    Parameters
    ----------
    topology : Topology
        The topology
    node : any hashable type
        The ID of the node
    name : str
        The name of the stack
    properties : dict, optional
        The properties of the stack
    **attr : keyworded attributes
        Further properties of the application
    """
    if properties is None:
        properties = {}
    elif not isinstance(properties, dict):
        raise TypeError('The properties parameter must be a dictionary')
    properties.update(kwargs)
    topology.node[node]['stack'] = name, properties


def get_stack(topology, node, data=True):
    """Return the stack of a node, if any

    Parameters
    ----------
    topology : Topology
        The topology
    node : any hashable type
        The ID of the node
    data : bool, optional
        If true, returns a tuple of the stack name and its attributes,
        otherwise just the stack name

    Returns
    -------
    stack : tuple (name, properties) or name only
        If data = True, a tuple of two values, where the first value is the
        name of the stack and the second value is the dictionary of its
        properties.
        If data = False returns only the stack name
        If no stack is deployed, return None
    """
    if 'stack' not in topology.node[node]:
        return None
    name, props = topology.node[node]['stack']
    return (name, props) if data else name


def remove_stack(topology, node):
    """Remove stack from a node

    Parameters
    ----------
    topology : Topology
        The topology
    node : any hashable type
        The ID of the node
    """
    topology.node[node].pop('stack', None)


def clear_stacks(topology):
    """Remove all stacks from all nodes of the topology

    Parameters
    ----------
    topology : Topology
    """
    for v in topology.nodes():
        topology.node[v].pop('stack', None)

def add_application(topology, node, name, properties=None, **attr):
    """Add an application to a node

    Parameters
    ----------
    topology : Topology
        The topology
    node : any hashable type
        The ID of the node
    name : str
        The name of the application
    attr_dict : dict, optional
        Attributes of the application
    **attr : keyworded attributes
        Attributes of the application
    """
    if properties is None:
        properties = {}
    elif not isinstance(properties, dict):
        raise TypeError('The attr_dict parameter must be a dictionary')
    properties.update(attr)
    if 'application' not in topology.node[node]:
        topology.node[node]['application'] = {}
    topology.node[node]['application'][name] = properties


def get_application_names(topology, node):
    """Return a list of names of applications deployed on a node

    Parameters
    ----------
    topology : Topology
        The topology
    node : any hashable type
        The ID of the node

    Returns
    -------
    application_names : list
        A list of application names
    """
    return [] if 'application' not in topology.node[node] \
              else list(topology.node[node]['application'].keys())


def get_application_properties(topology, node, name):
    """Return a dictionary containing all the properties of an application
    deployed on a node

    Parameters
    ----------
    topology : Topology
        The topology
    node : any hashable type
        The ID of the node
    name : str
        The name of the application

    Returns
    -------
    applications : dict
        A dictionary containing the properties of the application
    """
    if 'application' not in topology.node[node]:
        raise ValueError('There is no such application deployed on the node')
    return topology.node[node]['application'][name]


def remove_application(topology, node, name=None):
    """Remove an application from a node

    Parameters
    ----------
    topology : Topology
        The topology object
    node : any hashable type
        The ID of the node from which the application is to be removed
    name : optional
        The name of the application to remove. If not given, all the
        applications of the node are removed
    """
    if 'application' in topology.node[node]:
        if name is None:
            del topology.node[node]['application']
        elif name in topology.node[node]['application']:
            del topology.node[node]['application'][name]


def clear_applications(topology):
    """Remove all applications from all nodes of the topology

    Parameters
    ----------
    topology : Topology
        The topology
    """
    for v in topology.nodes():
        topology.node[v].pop('application', None)



import unittest

import networkx as nx

import fnss

class Test(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_delays_constant(self):
        topo = fnss.k_ary_tree_topology(3, 4)
        self.assertRaises(ValueError, fnss.set_delays_constant, topo, 2, 'Km')
        odd_links = [(u, v) for (u, v) in topo.edges()
                     if (u + v) % 2 == 1]
        even_links = [(u, v) for (u, v) in topo.edges()
                      if (u + v) % 2 == 0]
        fnss.set_delays_constant(topo, 2, 's', odd_links)
        fnss.set_delays_constant(topo, 5000000, 'us', even_links)
        self.assertEqual('s', topo.graph['delay_unit'])
        self.assertTrue(all(topo.adj[u][v]['delay'] in [2, 5]
                         for (u, v) in topo.edges()))

    def test_delays_geo_distance(self):
        specific_delay = 1.2
        L = 5
        G_len = fnss.waxman_1_topology(100, L=L)
        G_xy = fnss.waxman_2_topology(100, domain=(0, 0, 3, 4))
        # leave only node coordinate to trigger failure
        for u, v in G_xy.edges():
            del G_xy.adj[u][v]['length']
        self.assertRaises(ValueError, fnss.set_delays_geo_distance,
                          G_len, 2, delay_unit='Km')
        self.assertRaises(ValueError, fnss.set_delays_geo_distance,
                          G_xy, specific_delay, None, 'ms')
        fnss.set_delays_geo_distance(G_len, specific_delay,
                                None, 'ms', links=None)
        delays = nx.get_edge_attributes(G_len, 'delay')
        self.assertEqual(G_len.number_of_edges(), len(delays))
        self.assertGreaterEqual(specific_delay * L, max(delays.values()))
        self.assertLessEqual(0, min(delays.values()))

    def test_delays_geo_distance_conversions(self):
        topology = fnss.Topology(distance_unit='m')
        topology.add_edge(1, 2, length=2000)
        specific_delay = 1.2
        fnss.set_delays_geo_distance(topology, specific_delay, None, 'us')
        self.assertAlmostEqual(topology.adj[1][2]['delay'], 2400)
        fnss.clear_delays(topology)
        fnss.set_delays_geo_distance(topology, specific_delay, None, 's')
        self.assertAlmostEqual(topology.adj[1][2]['delay'], 0.0024)

    def test_delays_geo_distance_conversions_partial_assignments(self):
        topology = fnss.Topology(distance_unit='m')
        topology.add_edge(1, 2, length=2000)
        topology.add_edge(2, 3, length=3000)
        topology.add_edge(3, 4)
        specific_delay = 1.2
        fnss.set_delays_geo_distance(topology, specific_delay,
                                     None, 'us', links=[(1, 2)])
        fnss.set_delays_geo_distance(topology, specific_delay,
                                     3, 's', links=[(2, 3), (3, 4)])
        self.assertEquals(topology.graph['distance_unit'], 'm')
        self.assertEquals(topology.graph['delay_unit'], 'us')
        self.assertAlmostEqual(topology.adj[1][2]['delay'], 2400)
        self.assertAlmostEqual(topology.adj[2][3]['delay'], 3600)
        self.assertAlmostEqual(topology.adj[3][4]['delay'], 3000000)

    def test_delays_geo_distance_conversions_defaults(self):
        topology = fnss.Topology(distance_unit='m')
        topology.add_edge(1, 2, length=2000)
        topology.add_edge(2, 3, length=3000)
        topology.add_edge(3, 4)
        specific_delay = 1.2
        fnss.set_delays_geo_distance(topology, specific_delay, 3, 's', None)
        self.assertEquals(topology.graph['distance_unit'], 'm')
        self.assertEquals(topology.graph['delay_unit'], 's')
        self.assertAlmostEqual(topology.adj[1][2]['delay'], 0.0024)
        self.assertAlmostEqual(topology.adj[2][3]['delay'], 0.0036)
        self.assertAlmostEqual(topology.adj[3][4]['delay'], 3)

    def test_clear_delays(self):
        topo = fnss.star_topology(12)
        fnss.set_delays_constant(topo, 1, 'ms', None)
        self.assertEqual(topo.number_of_edges(),
                         len(nx.get_edge_attributes(topo, 'delay')))
        fnss.clear_delays(topo)
        self.assertEqual(0, len(nx.get_edge_attributes(topo, 'delay')))
